@@ -123,7 +123,7 @@ void RollTitle::setTitle( QString t )
 	p.fillRect( 0, 0, titlePix.width(), titlePix.height(), back );
 	p.setPen( fore );
 	p.setFont( font() );
-	p.drawText( QRect(0,(titlePix.height()-QFontInfo(font()).pixelSize())/2,titlePix.width(),QFontInfo(font()).pixelSize()), Qt::AlignAuto|Qt::SingleLine, t );
+	p.drawText( QRect(0,(titlePix.height()-QFontInfo(font()).pointSize())/2,titlePix.width(),QFontInfo(font()).pointSize()*2), Qt::AlignAuto|Qt::SingleLine, t );
 	p.end();
 }
 
@@ -311,7 +311,7 @@ PlayList::PlayList( QWidget* parent, QObject *objParent, const char *name ) : Ka
         else
             m_combo->lineEdit()->setText( "/" );*/
 
-	QVBoxLayout *v1 = new QVBoxLayout( vb, 3, 3 );
+	QVBoxLayout *v1 = new QVBoxLayout( 0, 3, 3 );
 	v1->addWidget( browserComb );
 	fileBrowser = new KDirOperator( KURL("/"), vb );
 	fileBrowser->setupMenu( KDirOperator::SortActions|KDirOperator::NavActions|KDirOperator::ViewActions );
@@ -387,6 +387,11 @@ PlayList::PlayList( QWidget* parent, QObject *objParent, const char *name ) : Ka
 	layout->addMultiCellWidget(coverFrame, 0, 2, 0, 0);
 
 	QHBoxLayout *h1 = new QHBoxLayout();
+	searchBtn = new QToolButton( playlist );
+	searchBtn->setAutoRaise( true );
+	QToolTip::add( searchBtn, i18n("Search"));
+	searchBtn->setIconSet( KGlobal::iconLoader()->loadIconSet("locationbar_erase", KIcon::Small) );
+	h1->addWidget( searchBtn );
 	QLabel* filterLabel = new QLabel(i18n("Filter") + ":", playlist);
 	h1->addWidget(filterLabel);
 	m_playlistFilter = new KLineEdit(playlist);
@@ -437,6 +442,7 @@ PlayList::PlayList( QWidget* parent, QObject *objParent, const char *name ) : Ka
 	connect(fileBrowser, SIGNAL(urlEntered( const KURL& )), this, SLOT(browserURLChanged( const KURL& )));
 	connect(fileBrowser, SIGNAL(fileSelected(const KFileItem*)), this, SLOT(fileSelected(const KFileItem*)));
 	connect(m_playlistFilter, SIGNAL(textChanged(const QString&)), this, SLOT(slotFindText(const QString&)));
+	connect( searchBtn, SIGNAL(clicked()), this, SLOT(resetSearch()) );
 	connect(m_playlistSelector, SIGNAL(returnPressed(const QString&)), this, SLOT(slotNewPlaylistName(const QString&)));
 	connect(m_playlistSelector, SIGNAL(activated(int)),this, SLOT(slotPlaylistActivated(int)));
 	connect(m_list, SIGNAL(dropped(QDropEvent*, QListViewItem*)), this, SLOT(slotDropEvent(QDropEvent*, QListViewItem*)));
@@ -453,6 +459,14 @@ PlayList::PlayList( QWidget* parent, QObject *objParent, const char *name ) : Ka
 	connect(m_list->header(), SIGNAL(clicked(int)), this, SLOT(slotSort(int)));
 }
 
+void PlayList::togglePanel()
+{
+	if ( panel->isHidden() )
+		panel->show();
+	else
+		panel->hide();
+}
+
 void PlayList::getTargets( QStringList &uiNames, QStringList &iconNames, QStringList &targetNames )
 {
 	uiNames.append( i18n("Play Playlist") );
@@ -489,6 +503,8 @@ void PlayList::setupActions()
 	m_shuffle->setStatusText(i18n("Play items in random order"));
 	m_autoCover = new KToggleAction(i18n("Autodownload covers"), 0 , 0, this, SLOT(slotAutoCover()), actionCollection(),"playlist_autocover");
 	m_autoCover->setStatusText(i18n("Automatic dowloading of covers"));
+	m_showPlayer = new KToggleAction(i18n("Don't switch to player window"), 0 , 0, this, SLOT(slotShowPlayer()), actionCollection(),"playlist_showplayer");
+	m_showPlayer->setStatusText(i18n("Don't switch automatically to player window"));
 	new KAction(i18n("&Clear Current Playlist"), "eraser", 0, this, SLOT(slotClearList()), actionCollection(), "playlist_clear");
 	new KAction(i18n("Ne&w Playlist"), "filenew", 0, this, SLOT(slotNewList()), actionCollection(), "playlist_new");
 	new KAction(i18n("&Import Playlist..."), "project_open", 0, this, SLOT(slotPlaylistLoad()), actionCollection(), "playlist_load");
@@ -516,6 +532,10 @@ void PlayList::slotAutoCover()
 	m_cover = m_autoCover->isChecked();
 }
 
+void PlayList::slotShowPlayer()
+{
+}
+
 void PlayList::slotToggleShuffle()
 {
 	m_shuffle->setChecked(!m_shuffle->isChecked());
@@ -585,6 +605,8 @@ void PlayList::loadConfig(KConfig* config)
 	b = config->readBoolEntry("AutoCover", false);
 	m_autoCover->setChecked(b);
 	m_cover = b;
+	b = config->readBoolEntry("DontShowPlayer", false);
+	m_showPlayer->setChecked(b);
 
 	sl = config->readIntListEntry("HSplitSizes");
 	if ( sl.count() )
@@ -634,6 +656,7 @@ void PlayList::saveConfig(KConfig* config)
 	config->writeEntry("Endless Mode", m_repeat->isChecked());
 	config->writeEntry("Random Mode", m_shuffle->isChecked());
 	config->writeEntry("AutoCover", m_autoCover->isChecked());
+	config->writeEntry("DontShowPlayer", m_showPlayer->isChecked());
 	config->writeEntry( "HSplitSizes", hSplit->sizes() );
 	config->writeEntry( "VSplitSizes", vSplit->sizes() );
 	QStringList list;
@@ -997,7 +1020,7 @@ void PlayList::add(const QStringList& urlList, QListViewItem* after)
 	KURL tmpKURL;
 	bool playlist;
 	MRL mrl;
-	MRL::List mrlList;
+	QValueList<MRL> mrlList;
 	QString mediadevice="";
 
 	KProgressDialog* progress = NULL;
@@ -1065,6 +1088,7 @@ void PlayList::add(const QStringList& urlList, QListViewItem* after)
 			if (!QFile::exists(mrl.kurl().path()))
 				continue;
 			mrl.setTitle(mrl.kurl().fileName());
+			mrl.setURL(mrl.kurl().path());
 		}
 		else
 		{
@@ -1135,8 +1159,8 @@ void PlayList::add(const QStringList& urlList, QListViewItem* after)
 				}
 				if (playlist)
 				{
-					MRL::List::ConstIterator end(mrlList.end());
-					for (MRL::List::ConstIterator it = mrlList.begin(); it != end; ++it)
+					QValueList<MRL>::ConstIterator end(mrlList.end());
+					for (QValueList<MRL>::ConstIterator it = mrlList.begin(); it != end; ++it)
 						after = insertItem(after, *it);
 					continue;
 				}
@@ -1150,8 +1174,8 @@ void PlayList::add(const QStringList& urlList, QListViewItem* after)
 				kdDebug() << "PlayList: Try loading ram playlist\n";
 				if (PlaylistImport::ram(mrl, mrlList, mainWidget->parentWidget()))
 				{
-					MRL::List::ConstIterator end(mrlList.end());
-					for (MRL::List::ConstIterator it = mrlList.begin(); it != end; ++it)
+					QValueList<MRL>::ConstIterator end(mrlList.end());
+					for (QValueList<MRL>::ConstIterator it = mrlList.begin(); it != end; ++it)
 						after = insertItem(after, *it);
 					continue;
 				}
@@ -1328,10 +1352,10 @@ void PlayList::add(const QStringList& urlList, QListViewItem* after)
 	updateStatus();
 }
 
-void PlayList::add(const MRL::List& mrlList, QListViewItem* after)
+void PlayList::add(const QValueList<MRL>& mrlList, QListViewItem* after)
 {
-	MRL::List::ConstIterator end(mrlList.end());
-	for (MRL::List::ConstIterator it = mrlList.begin(); it != end; ++it)
+	QValueList<MRL>::ConstIterator end(mrlList.end());
+	for (QValueList<MRL>::ConstIterator it = mrlList.begin(); it != end; ++it)
 		after = insertItem(after, *it);
 	updateStatus();
 	if (m_random) createRandomList();
@@ -1411,9 +1435,11 @@ void PlayList::slotDropEvent(QDropEvent* dev, QListViewItem* after)
 		{
 			for (uint i=0; i < urls.count() ;i++)
 			{
-				KURL url(QUriDrag::unicodeUriToUri(urls[i]));
-				newurls << url.path(-1);
-				kdDebug() << "PlayList: Dropped " << url.path() << endl;
+				//KURL url(QUriDrag::unicodeUriToUri(urls[i]));
+				//newurls << url.path(-1);
+				//kdDebug() << "PlayList: Dropped " << url.path() << endl;
+				newurls << urls[i];
+				kdDebug() << "PlayList: Dropped " << urls[i] << endl;
 			}
 			add(newurls, after);
 		}
@@ -1745,8 +1771,8 @@ void PlayList::updateStatus()
 	if (isQueueMode())
 	{
 		QTime total;
-		MRL::List::ConstIterator end(m_queue.end());
-		for (MRL::List::ConstIterator it = m_queue.begin(); it != end; ++it)
+		QValueList<MRL>::ConstIterator end(m_queue.end());
+		for (QValueList<MRL>::ConstIterator it = m_queue.begin(); it != end; ++it)
 			total = total.addSecs(QTime().secsTo((*it).length()));
 		status = i18n("Queue: %1 Entries, Playtime: %2").arg(m_queue.count()).arg(total.toString("h:mm:ss"));
 	}
@@ -2050,6 +2076,12 @@ void PlayList::exportPLSPlaylist(const QString& pl)
 	file.close();
 }
 
+void PlayList::resetSearch()
+{
+	m_playlistFilter->clear();
+	slotFindText( "" );
+}
+
 void PlayList::slotFindText(const QString& text)
 {
 	if (text == i18n("Filter")) return;
@@ -2160,7 +2192,7 @@ void PlayList::slotPlaylistFromSelected()
 	QPtrList<QListViewItem> selected;
 	selected = m_list->selectedItems();
 
-	MRL::List mrlList;
+	QValueList<MRL> mrlList;
 
 	for (uint i=0; i<selected.count(); i++)
 	{
