@@ -23,8 +23,6 @@
 #include <unistd.h>
 #include <errno.h>
 
-#include <linux/dvb/frontend.h>
-
 #include <qdir.h>
 #include <qlayout.h>
 #include <qgroupbox.h>
@@ -39,7 +37,6 @@
 #include <kstandarddirs.h>
 
 #include "dvbconfig.h"
-#include "gdvb.h"
 #include "kaffeinedvbplugin.h"
 
 
@@ -155,34 +152,27 @@ DVBconfig::~DVBconfig()
 
 bool DVBconfig::haveDvbDevice()
 {
-	int i=0, j=0, res, fdFrontend=0;
+	int i, j, res, fdFrontend=0;
 	struct dvb_frontend_info info;
-	fprintf(stderr,"%d\n", errno);
 
-	QStringList list;
+	QStringList list, flist;
 	QDir d;
 	d.setPath( "/dev/dvb/" );
 	list = d.entryList( "adapter*", QDir::Dirs, QDir::Name );
 
 	for ( i=0; i<(int)list.count(); i++ ) {
-		j = 0;
-		while ( j>=0 && j<4 ) {
-			fdFrontend = open( QString("/dev/dvb/%1/frontend%2").arg( list[i] ).arg( j ).ascii(), O_RDWR);
-			if ( fdFrontend<0 ) {
-				if ( errno==ENOENT )
-					j=-1;
-				else
-					++j;
-			}
-			else {
+		d.setPath( "/dev/dvb/"+list[i]+"/" );
+		flist = d.entryList( "frontend*", QDir::System, QDir::Name );
+		for ( j=0; j<(int)flist.count(); j++ ) {
+			fdFrontend = open( QString("/dev/dvb/%1/%2").arg( list[i] ).arg( flist[j] ).ascii(), O_RDWR);
+			if ( fdFrontend>0 ) {
 				if ( (res = ioctl( fdFrontend, FE_GET_INFO, &info ) < 0) )
-					perror( QString("/dev/dvb/%1/frontend%2 FE_GET_INFO: ").arg( list[i] ).arg( j ).ascii() );
+					perror( QString("/dev/dvb/%1/%2 FE_GET_INFO: ").arg( list[i] ).arg( flist[j] ).ascii() );
 				else {
 					close( fdFrontend );
 					return true;
 				}
 				close( fdFrontend );
-				++j;
 			}
 		}
 	}
@@ -198,23 +188,20 @@ void DVBconfig::startup()
 	struct dvb_frontend_info info;
 	bool as;
 
-	QStringList list;
+	QStringList list, flist;
+	QString s, t;
 	QDir d;
 	d.setPath( "/dev/dvb/" );
 	list = d.entryList( "adapter*", QDir::Dirs, QDir::Name );
 
 	for ( i=0; i<(int)list.count(); i++ ) {
-		j = 0;
-		while ( j>=0 && j<4 ) {
-			fdFrontend = open( QString("/dev/dvb/%1/frontend%2").arg( list[i] ).arg( j ).ascii(), O_RDWR);
-			if ( fdFrontend<0 ) {
-				perror( QString("/dev/dvb/%1/frontend%2 : ").arg( list[i] ).arg( j ).ascii() );
-				if ( errno==ENOENT )
-					j=-1;
-				else
-					++j;
-			}
-			else {
+		d.setPath( "/dev/dvb/"+list[i]+"/" );
+		flist = d.entryList( "frontend*", QDir::System, QDir::Name );
+		for ( j=0; j<(int)flist.count(); j++ ) {
+			s = list[i];
+			t = flist[j];
+			fdFrontend = open( QString("/dev/dvb/%1/%2").arg( s ).arg( t ).ascii(), O_RDWR);
+			if ( fdFrontend>0 ) {
 				if ( !(res = ioctl( fdFrontend, FE_GET_INFO, &info ) < 0) ) {
 					if ( (info.type==FE_OFDM)
 						&& (info.caps & FE_CAN_QAM_AUTO)
@@ -225,19 +212,17 @@ void DVBconfig::startup()
 						as = true;
 					else
 						as = false;
-					fprintf(stderr,"/dev/dvb/%s/frontend%d : opened ( %s )\n", list[i].ascii(), j, info.name );
-					devList.append( new Device( list[i].replace("adapter","").toInt(), j, info.type, info.name, as ) );
+					fprintf(stderr,"/dev/dvb/%s/%s : opened ( %s )\n", s.ascii(), t.ascii(), info.name );
+					devList.append( new Device( s.replace("adapter","").toInt(), t.replace("frontend","").toInt(), info.type, info.name, as ) );
 				}
 				close( fdFrontend );
-				++j;
 			}
+			else
+				perror( QString("/dev/dvb/%1/%2").arg( s ).arg( t ).ascii() );
 		}
 	}
 
-	//devList.append( new Device( 0, 1, FE_QPSK, "Dummy S", false ) );
-	//devList.append( new Device( 1, 1, FE_OFDM, "Dummy T", false ) );
-	//devList.append( new Device( 2, 0, FE_QAM, "Dummy C", false ) );
-	//devList.append( new Device( 3, 0, FE_ATSC, "Dummy A", false ) );
+	//devList.append( new Device( 3, 0, FE_QPSK, "Dummy S", false ) );
 }
 
 
@@ -417,7 +402,6 @@ void DVBconfig::readConfig()
 	beginMargin = config->readNumEntry( "BeginMargin", 5 );
 	endMargin = config->readNumEntry( "EndMargin", 10 );
 	instantDuration = config->readNumEntry( "InstantDuration", 120 );
-	format = config->readNumEntry( "PrefFormat", OutTS );
 	recordDir = config->readEntry( "RecordDir", QDir::homeDirPath() );
 	if ( !recordDir.endsWith("/") )
 		recordDir+= "/";
@@ -425,6 +409,7 @@ void DVBconfig::readConfig()
 	shiftDir = config->readEntry( "ShiftDir", QDir::homeDirPath() );
 	if ( !shiftDir.endsWith("/") )
 		shiftDir+= "/";
+	filenameFormat = config->readEntry( "filenameFormat", "%name" );
 	for ( i=0; i<(int)devList.count(); i++ ) {
 		devList.at(i)->source = config->readEntry( QString("DVB%1").arg(i), "" );
 		devList.at(i)->tuningTimeout = config->readNumEntry( QString("DVB%1_TIMEOUT").arg(i), 1500 );
@@ -472,9 +457,9 @@ void DVBconfig::saveConfig()
 	config->writeEntry( "BeginMargin", beginMargin );
 	config->writeEntry( "EndMargin", endMargin );
 	config->writeEntry( "InstantDuration", instantDuration );
-	config->writeEntry( "PrefFormat", format );
 	config->writeEntry( "RecordDir", recordDir );
 	config->writeEntry( "ShiftDir", shiftDir );
+	config->writeEntry( "filenameFormat", filenameFormat );
 	config->writeEntry( "BroadcastAddress", broadcastAddress );
 	config->writeEntry( "BroadcastPort", broadcastPort );
 	config->writeEntry( "SenderPort", senderPort );
@@ -808,20 +793,23 @@ DvbConfigDialog::DvbConfigDialog( DVBconfig *dc, QWidget *parent, KaffeineDvbPlu
 	h1->addWidget( lab );
 	grid->addLayout( h1, 4, 1 );
 
-	lab = new QLabel( i18n("Preferred format:"), gb );
-	grid->addWidget( lab, 5, 0 );
-	formatComb = new QComboBox( gb );
-	grid->addWidget( formatComb, 5, 1 );
-
 	lab = new QLabel( i18n("Max file size (0=Unlimited):"), gb );
-	grid->addWidget( lab, 6, 0 );
+	grid->addWidget( lab, 5, 0 );
 	sizeFileSpin = new QSpinBox( 1, 9999, 1, gb );
 	sizeFileSpin->setMinValue(0);
 	h1 = new QHBoxLayout();
 	h1->addWidget( sizeFileSpin );
 	lab = new QLabel( i18n("(MB)"), gb );
 	h1->addWidget( lab );
-	grid->addLayout( h1, 6, 1 );
+	grid->addLayout( h1, 5, 1 );
+
+	lab = new QLabel( i18n("Filename Format:"), gb );
+	grid->addWidget( lab, 7, 0 );
+	filenameFormatLe = new QLineEdit( gb );
+	grid->addWidget( filenameFormatLe, 7, 1 );
+	helpNameBtn = new QToolButton( gb );
+	helpNameBtn->setIconSet( icon->loadIconSet("help", KIcon::Small) );
+	grid->addWidget( helpNameBtn, 7, 2 );
 
 	vb->addWidget( gb );
 	vb->addItem( new QSpacerItem( 20, 20, QSizePolicy::Ignored, QSizePolicy::Ignored ) );
@@ -832,14 +820,7 @@ DvbConfigDialog::DvbConfigDialog( DVBconfig *dc, QWidget *parent, KaffeineDvbPlu
 	endSpin->setValue( dvbConfig->endMargin );
 	instantDurationSpin->setValue( dvbConfig->instantDuration );
 	sizeFileSpin->setValue( dvbConfig->sizeFile );
-	formatComb->insertItem( "TS" );
-	formatComb->insertItem( "MPEG_PES" );
-	formatComb->insertItem( "MPEG_PS" );
-	switch ( dvbConfig->format ) {
-		case OutTS : formatComb->setCurrentItem( 0 ); break;
-		case OutPES : formatComb->setCurrentItem( 1 ); break;
-		case OutPS : formatComb->setCurrentItem( 2 ); break;
-	}
+	filenameFormatLe->setText( dvbConfig->filenameFormat );
 
 	page = addPage(i18n("Broadcasting"),i18n("DVB Broadcasting"),
 		KGlobal::instance()->iconLoader()->loadIcon( "network_local", KIcon::NoGroup, KIcon::SizeMedium ) );
@@ -916,6 +897,7 @@ DvbConfigDialog::DvbConfigDialog( DVBconfig *dc, QWidget *parent, KaffeineDvbPlu
 	connect( recordDirBtn, SIGNAL(clicked()), this, SLOT(setRecordDir()) );
 	connect( shiftDirBtn, SIGNAL(clicked()), this, SLOT(setShiftDir()) );
 	connect( updateBtn, SIGNAL(clicked()), this, SLOT(downloadData()) );
+	connect( helpNameBtn, SIGNAL(clicked()), this, SLOT(fileTemplateHelp()) );
 	delete icon;
 }
 
@@ -1061,6 +1043,13 @@ void DvbConfigDialog::satNumberChanged( int value, int devNum )
 
 
 
+void DvbConfigDialog::fileTemplateHelp()
+{
+	KMessageBox::information( this, i18n("Special strings are:\n- %chan (channel's name)\n- %date (the starting date, YYMMdd-hhmmss)\n- %name (the name given in timer editor or the program name from EPG)\nSo, if you set template to '%chan-%date-%name', the resulting filename will be, for example, BBC-20070919-210000-News.m2t") );
+}
+
+
+
 void DvbConfigDialog::setRecordDir()
 {
 	QString s = KFileDialog::getExistingDirectory( recordDirLe->text().stripWhiteSpace() );
@@ -1104,6 +1093,11 @@ void DvbConfigDialog::accept()
 		broadcastLe->setFocus();
 		return;
 	}
+	if ( !filenameFormatLe->text().contains("%chan") && !filenameFormatLe->text().contains("%date") && !filenameFormatLe->text().contains("%name") ) {
+		KMessageBox::sorry( this, i18n("Invalid filename format.") );
+		filenameFormatLe->setFocus();
+		return;
+	}
 
 	for ( i=0; i<(int)dvbConfig->devList.count(); i++ ) {
 		switch (dvbConfig->devList.at(i)->type) {
@@ -1158,11 +1152,7 @@ void DvbConfigDialog::accept()
 	dvbConfig->endMargin = endSpin->value();
 	dvbConfig->instantDuration = instantDurationSpin->value();
 	dvbConfig->sizeFile = sizeFileSpin->value();
-	switch ( formatComb->currentItem() ) {
-		case 0 : dvbConfig->format = OutTS; break;
-		case 1 : dvbConfig->format = OutPES; break;
-		case 2 : dvbConfig->format = OutPS; break;
-	}
+	dvbConfig->filenameFormat = filenameFormatLe->text();
 	dvbConfig->defaultCharset = charsetComb->currentText();
 	dvbConfig->broadcastAddress = broadcastLe->text().stripWhiteSpace();
 	dvbConfig->broadcastPort = bportSpin->value();
