@@ -1,7 +1,7 @@
 /*
  * dvbout.cpp
  *
- * Copyright (C) 2004-2006 Christophe Thommeret <hftom@free.fr>
+ * Copyright (C) 2004-2007 Christophe Thommeret <hftom@free.fr>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,6 +18,8 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
  */
 
+#include <fcntl.h>
+
 #include <kstandarddirs.h>
 
 #include "dvbout.h"
@@ -29,7 +31,7 @@ DVBout::DVBout( ChannelDesc chan, int anum, int tnum, KaffeineDvbPlugin *p )
 {
 	bool bok;
 	bok = true;
-	unsigned int i=anum, j=tnum, k;
+	unsigned int i, j=0, k;
 
 	for( i = 0 ; i < 256 ; i++ ) {
 		k = 0;
@@ -46,20 +48,22 @@ DVBout::DVBout( ChannelDesc chan, int anum, int tnum, KaffeineDvbPlugin *p )
 	}
 
 	fdPipe=0;
-	outType = 0;
 	channel = chan;
 	thWrite = 0;
-	tp = 0;
-	tpShift = 0;
 	rtp = 0;
-	activeApid = 0;
+	if ( channel.vpid )
+		pids.append( channel.vpid );
+	for ( i=0; i<channel.napid && i<MAX_AUDIO; i++ )
+		pids.append( channel.apid[i].pid );
+	for ( i=0; i<channel.nsubpid && i<MAX_DVBSUB; i++ )
+		pids.append( channel.subpid[i].pid );
 	wBuf = NULL;
 	timeShifting = beginLive = false;
 	haveRec = haveLive = instantRec = haveBroadcast = false;
 	patpmt = wpatpmt = false;
 	connect( &stopRecTimer, SIGNAL(timeout()), this, SLOT(stopRec()) );
 	connect( &timerPatPmt, SIGNAL(timeout()), this, SLOT(setPatPmt()) );
-	if ( channel.apid[activeApid].pid!=8192)
+	if ( !pids.contains(8192) )
 		timerPatPmt.start(500);
 }
 
@@ -92,7 +96,7 @@ void DVBout::writePat()
 	tspat[0x01] = 0x40;
 	tspat[0x02] = 0x00; // PID = 0x0000
 	tspat[0x03] = 0x10; // | (ps->pat_counter & 0x0f);
-	tspat[0x04] = 0x00; // CRC calculation begins here
+	tspat[0x04] = 0x00; // pointer_field. CRC calculation begins here
 	tspat[0x05] = 0x00; // 0x00: Program association section
 	tspat[0x06] = 0xb0;
 	tspat[0x07] = 0x11; // section_length = 0x011
@@ -104,8 +108,8 @@ void DVBout::writePat()
 	// Network PID (useless)
 	tspat[0x0d] = tspat[0x0e] = 0x00; tspat[0x0f] = 0xe0; tspat[0x10] = 0x10;
 	// Program Map PID
-	pmtpid = 0x64;
-	while ( pmtpid==channel.vpid || pmtpid==channel.apid[activeApid].pid || pmtpid==channel.subpid[0].pid ) pmtpid--;
+	pmtpid = 0xff;
+	while ( pids.contains( pmtpid ) ) pmtpid--;
 	tspat[0x11] = 0x03; tspat[0x12] = 0xe8; tspat[0x13] = 0xe0; tspat[0x14] = pmtpid;
 	// Put CRC in ts[0x15...0x18]
 	calculateCRC( tspat + 0x05, tspat + 0x15 );
@@ -123,7 +127,7 @@ void DVBout::writePmt()
 	tspmt[0x01] = 0x40;
 	tspmt[0x02] = pmtpid;
 	tspmt[0x03] = 0x10;
-	tspmt[0x04] = 0x00; // CRC calculation begins here
+	tspmt[0x04] = 0x00; // pointer_field. CRC calculation begins here
 	tspmt[0x05] = 0x02; // 0x02: Program map section
 	tspmt[0x06] = 0xb0;
 	tspmt[0x07] = 0x20; // section_length
@@ -145,52 +149,63 @@ void DVBout::writePmt()
 	tspmt[0x1a] = 0x24; tspmt[0x1b] = 0x02; tspmt[0x1c] = 0x11; tspmt[0x1d] = 0x01;
 	tspmt[0x1e] = 0xfe;
 	off = 0x1e;
-	// audio pid
-	if ( channel.apid[activeApid].ac3 ) {
-		tspmt[++off] = 0x81; // stream type = xine see this as ac3
-		tspmt[++off] = channel.apid[activeApid].pid>>8; tspmt[++off] = channel.apid[activeApid].pid&0xff;
-		tspmt[++off] = 0xf0; tspmt[++off] = 0x0c; // es info length
-		tspmt[++off] = 0x05; tspmt[++off] = 0x04; tspmt[++off] = 0x41;
-		tspmt[++off] = 0x43; tspmt[++off] = 0x2d; tspmt[++off] = 0x33;
+	// audio pids
+	i = 0;
+	for ( i=0; i<channel.napid && i<MAX_AUDIO; i++ ) {
+		if ( channel.apid[i].ac3 ) {
+			tspmt[++off] = 0x81; // stream type = xine see this as ac3
+			tspmt[++off] = channel.apid[i].pid>>8; tspmt[++off] = channel.apid[i].pid&0xff;
+			tspmt[++off] = 0x00; tspmt[++off] = 0x0c; // es info length
+			tspmt[++off] = 0x05; tspmt[++off] = 0x04; tspmt[++off] = 0x41;
+			tspmt[++off] = 0x43; tspmt[++off] = 0x2d; tspmt[++off] = 0x33;
+		}
+		else {
+			tspmt[++off] = 0x04; // stream type = audio
+			tspmt[++off] = channel.apid[i].pid>>8; tspmt[++off] = channel.apid[i].pid&0xff;
+			tspmt[++off] = 0xf0; tspmt[++off] = 0x06; // es info length
+		}
+		tspmt[++off] = 0x0a; // iso639 descriptor tag
+		tspmt[++off] = 0x04; // descriptor length
+		if ( !channel.apid[i].lang.isEmpty() ) {
+			tspmt[++off] = channel.apid[i].lang.constref(0);
+			tspmt[++off] = channel.apid[i].lang.constref(1);
+			if ( channel.apid[i].ac3 )
+				tspmt[++off] = '_';
+			else
+				tspmt[++off] = channel.apid[i].lang.constref(2);
+		}
+		else if ( channel.apid[i].ac3 ) {
+			tspmt[++off] = 'd';
+			tspmt[++off] = 'd';
+			tspmt[++off] = 49+i;
+		}
+		else {
+			tspmt[++off] = 'c';
+			tspmt[++off] = 'h';
+			tspmt[++off] = 49+i;
+		}
+		tspmt[++off] = 0x00; // audio type
 	}
-	else {
-		tspmt[++off] = 0x04; // stream type = audio
-		tspmt[++off] = channel.apid[activeApid].pid>>8; tspmt[++off] = channel.apid[activeApid].pid&0xff;
-		tspmt[++off] = 0xf0; tspmt[++off] = 0x06; // es info length
-	}
-	tspmt[++off] = 0x0a; // iso639 descriptor tag
-	tspmt[++off] = 0x04; // descriptor length
-	if ( !channel.apid[activeApid].lang.isEmpty() ) {
-		tspmt[++off] = channel.apid[activeApid].lang.constref(0);
-		tspmt[++off] = channel.apid[activeApid].lang.constref(1);
-		tspmt[++off] = channel.apid[activeApid].lang.constref(2);
-	}
-	else {
-		tspmt[++off] = '?';
-		tspmt[++off] = '?';
-		tspmt[++off] = '?';
-	}
-	tspmt[++off] = 0x00; // audio type
 	// Subtitles
-	if ( channel.subpid[0].pid ) {
+	for ( i=0; i<channel.nsubpid && i<MAX_DVBSUB; i++ ) {
 		tspmt[++off] = 0x06; // stream type = ISO_13818_PES_PRIVATE
-		tspmt[++off] = channel.subpid[0].pid>>8; tspmt[++off] = channel.subpid[0].pid&0xff;
+		tspmt[++off] = channel.subpid[i].pid>>8; tspmt[++off] = channel.subpid[i].pid&0xff;
 		tspmt[++off] = 0xf0; tspmt[++off] = 0x0a; // es info length
 		tspmt[++off] = 0x59; //DVB sub tag
 		tspmt[++off] = 0x08; // descriptor length
-		if ( !channel.subpid[0].lang.isEmpty() ) {
-			tspmt[++off] = channel.subpid[0].lang.constref(0);
-			tspmt[++off] = channel.subpid[0].lang.constref(1);
-			tspmt[++off] = channel.subpid[0].lang.constref(2);
+		if ( !channel.subpid[i].lang.isEmpty() ) {
+			tspmt[++off] = channel.subpid[i].lang.constref(0);
+			tspmt[++off] = channel.subpid[i].lang.constref(1);
+			tspmt[++off] = channel.subpid[i].lang.constref(2);
 		}
 		else {
-			tspmt[++off] = '?';
-			tspmt[++off] = '?';
-			tspmt[++off] = '?';
+			tspmt[++off] = 'c';
+			tspmt[++off] = 'h';
+			tspmt[++off] = 49+i;
 		}
-		tspmt[++off] = channel.subpid[0].type; //sub type
-		tspmt[++off] = channel.subpid[0].page>>8; tspmt[++off] = channel.subpid[0].page&0xff; // comp_page_id
-		tspmt[++off] = channel.subpid[0].id>>8; tspmt[++off] = channel.subpid[0].id&0xff; // anc_page_id
+		tspmt[++off] = channel.subpid[i].type; //sub type
+		tspmt[++off] = channel.subpid[i].page>>8; tspmt[++off] = channel.subpid[i].page&0xff; // comp_page_id
+		tspmt[++off] = channel.subpid[i].id>>8; tspmt[++off] = channel.subpid[i].id&0xff; // anc_page_id
 	}
 	tspmt[0x07] = off-3; // update section_length
 	// Put CRC in ts[0x29...0x2c]
@@ -201,13 +216,6 @@ void DVBout::writePmt()
 
 
 
-int DVBout::currentAudioPid() const
-{
-	return activeApid;
-}
-
-
-
 bool DVBout::hasInstantRec() const
 {
 	return instantRec;
@@ -278,18 +286,18 @@ void DVBout::setPatPmt()
 
 
 
-bool DVBout::goLive( const QString &name, int napid )
+bool DVBout::goLive( const QString &name )
 {
 	if ( fdPipe ) return false;
 
 	haveLive = true;
 	pipeName = name;
 	beginLive = true;
-	activeApid = napid;
+	//activeApid = napid;
 
 	writePat();
 	writePmt();
-	if ( channel.apid[activeApid].pid!=8192 )
+	if ( !pids.contains(8192) )
 		patpmt = wpatpmt = true;
 	wBuf = new unsigned char[TS_SIZE*NTS*100];
 	if ( !wBuf ) fprintf( stderr, "\nNO WBUF !!!\n\n" );
@@ -336,10 +344,8 @@ void DVBout::stopLive()
 bool DVBout::goBroadcast( Ts2Rtp *r )
 {
 	if ( haveBroadcast ) return false;
-	fprintf(stderr,"Start Broadcast: %s\n", channel.name.latin1() );
+	fprintf(stderr,"Start Broadcast: %s\n", channel.name.ascii() );
 	rtp = r;
-	rtp->addChannel( channel.name, channel.vpid, channel.apid[activeApid].pid, channel.apid[activeApid].ac3,
-		channel.subpid[0].pid, channel.subpid[0].page, channel.subpid[0].id, channel.subpid[0].type, channel.subpid[0].lang );
 	haveBroadcast = true;
 	return true;
 }
@@ -349,60 +355,47 @@ bool DVBout::goBroadcast( Ts2Rtp *r )
 void DVBout::stopBroadcast()
 {
 	if ( !haveBroadcast ) return;
-	fprintf(stderr,"Stop Broadcast: %s\n", channel.name.latin1() );
+	fprintf(stderr,"Stop Broadcast: %s\n", channel.name.ascii() );
 	mutex.lock();
-	rtp->removeChannel( channel.name );
+	rtp->removeChannels();
 	haveBroadcast = false;
 	mutex.unlock();
 }
 
 
 
-bool DVBout::goRec( int type, const QString &name, RecTimer *t )
+bool DVBout::goRec( const QString &name, int maxsize, RecTimer *t )
 {
-	int ps = 0;
-	bool ok;
 	QString fname=name;
 
+	if ( haveRec )
+		return false;
+
 	recTimer = t;
+	fileName=name;
+	fileNumber=0;
 
-	if ( outType ) return false;
-	if ( tp ) return false;
-
-	if ( channel.apid[activeApid].ac3 || !channel.vpid || channel.subpid[0].pid || channel.apid[activeApid].pid==8192 )
-		type = OutTS;
-
-	switch ( type ) {
-		case OutPS : ps = 1;
-		case OutPES : {
-			if ( QFile(fname+".mpg").exists() )
-				renameFile( fname, ".mpg" );
-			tp = new Ts2Pes( fname, channel.apid[0].pid, channel.vpid, ps, &ok );
-			if ( !ok ) {
-				delete tp;
-				tp = 0;
-				return false;
-			}
-			tp->go();
-			recTimer->fullPath = fname+".mpg";
-			break;
-		}
-		case OutTS : {
-			if ( QFile(fname+".m2t").exists() )
-				renameFile( fname, ".m2t" );
-			writePat();
-			writePmt();
-			if ( channel.apid[activeApid].pid!=8192 )
-				patpmt = true;
-			outFile.setName( fname+".m2t" );
-			if ( !outFile.open( IO_WriteOnly | IO_Truncate ) ) return false;
-			outFile.writeBlock( (char*)tspat, TS_SIZE );
-			outFile.writeBlock( (char*)tspmt, TS_SIZE );
-			recTimer->fullPath = fname+".m2t";
-			break;
-		}
+	if (maxsize>0) {
+		fileMaxSize = (long long int)1048576*(long long int)maxsize;
+		fname=fileName+"_"+QString().setNum(fileNumber);
 	}
-	outType = type;
+	else {
+		fileMaxSize = 0;
+		fname=fileName;
+	}
+	if ( QFile(fname+".m2t").exists() )
+		renameFile( fname, ".m2t" );
+	writePat();
+	writePmt();
+	if ( channel.apid[0].pid!=8192 )
+		patpmt = true;
+	outFile.setName( fname+".m2t" );
+	if ( !outFile.open( IO_WriteOnly | IO_Truncate ) )
+		return false;
+	outFile.writeBlock( (char*)tspat, TS_SIZE );
+	outFile.writeBlock( (char*)tspmt, TS_SIZE );
+	recTimer->fullPath = fname+".m2t";
+
 	haveRec = true;
 	if ( recTimer ) {
 		QTime t = recTimer->duration.addSecs( QDateTime::currentDateTime().secsTo(recTimer->begin) );
@@ -438,21 +431,13 @@ void DVBout::changeTimer( int ms )
 
 void DVBout::stopRec()
 {
-	if ( !(outType) ) return;
+	if ( !haveRec )
+		return;
 
 	if ( stopRecTimer.isActive() )
 		stopRecTimer.stop();
 	mutex.lock();
-	outType=0;
-	switch (outType) {
-		case OutPES :
-		case OutPS : {
-			delete tp;
-			tp = 0;
-			break;
-		}
-		case OutTS : outFile.close();
-	}
+	outFile.close();
 	mutex.unlock();
 	haveRec = instantRec = false;
 	fprintf( stderr, "Recording stopped: %s\n", channel.name.latin1() );
@@ -468,10 +453,11 @@ void DVBout::process( unsigned char *buf, int size )
 {
 	int i, pid;
 	unsigned char *buffer=buf;
+	QString fname;
 
 	for ( i=0; i<size; i+=TS_SIZE ) {
 		pid = (((buffer[1] & 0x1f) << 8) | buffer[2]);
-		if ( channel.vpid==pid || channel.apid[activeApid].pid==pid || channel.subpid[0].pid==pid || channel.apid[activeApid].pid==8192) {
+		if ( pids.contains( pid ) || pids.contains( 8192) ) {
 			memcpy( thBuf+thWrite, buffer, TS_SIZE );
 			thWrite+=TS_SIZE;
 			if ( thWrite==(TS_SIZE*NTS ) ) {
@@ -500,14 +486,33 @@ void DVBout::process( unsigned char *buf, int size )
 					}
 					liveFile.writeBlock( (char*)thBuf, TS_SIZE*NTS );
 				}
-				if ( outType > OutTS )
-					tp->run( thBuf, TS_SIZE*NTS );
-				else if ( outType==OutTS ) {
-					if ( patpmt ) {
-						outFile.writeBlock( (char*)tspat, TS_SIZE );
-						outFile.writeBlock( (char*)tspmt, TS_SIZE );
+				if ( haveRec && fileNumber>=0 ) {
+					if ((fileMaxSize>0)&&(outFile.size()>=fileMaxSize)) {
+						outFile.close();
+						fileNumber++;
+						fname=fileName+"_"+QString().setNum(fileNumber);
+						if (QFile(fname+"_"+QString().setNum(fileNumber)+".m2t").exists() )
+							renameFile( fname, ".m2t" );
+
+						outFile.setName( fname+".m2t" );
+						if ( !outFile.open( IO_WriteOnly | IO_Truncate ) )
+							fileNumber=-1;
+						else
+						{
+							outFile.writeBlock( (char*)tspat, TS_SIZE );
+							outFile.writeBlock( (char*)tspmt, TS_SIZE );
+							recTimer->fullPath = fname+".m2t";
+						}
 					}
-					outFile.writeBlock( (char*)thBuf, TS_SIZE*NTS );
+					if (fileNumber>=0)
+					{
+						if ( patpmt ) {
+							outFile.writeBlock( (char*)tspat, TS_SIZE );
+							outFile.writeBlock( (char*)tspmt, TS_SIZE );
+						}
+						outFile.writeBlock( (char*)thBuf, TS_SIZE*NTS );
+					}
+
 				}
 				if ( haveBroadcast )
 					rtp->process( thBuf, TS_SIZE*NTS );
