@@ -1,19 +1,24 @@
-/***************************************************************************
-                          dvbpanel.cpp  -  description
-                             -------------------
-    begin                : Mon Jan 19 2004
-    copyright            : (C) 2004-2006 by Christophe Thommeret
-    email                : hftom@free.fr
- ***************************************************************************/
+/*
+ * dvbpanel.cpp
+ *
+ * Copyright (C) 2004-2007 Christophe Thommeret <hftom@free.fr>
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
+ */
 
-/***************************************************************************
- *                                                                         *
- *   This program is free software; you can redistribute it and/or modify  *
- *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
- *   (at your option) any later version.                                   *
- *                                                                         *
- ***************************************************************************/
+#include <sys/statvfs.h>
 
 #include <qlayout.h>
 #include <qfile.h>
@@ -45,9 +50,9 @@
 #include "dvbpanel.h"
 #include "channeldesc.h"
 #include "dvbstream.h"
-#include "dvbevents.h"
 #include "kevents.h"
 #include "broadcasteditor.h"
+#include "channeleditor.h"
 
 #define CHANICONSIZE 28
 
@@ -75,6 +80,7 @@ void DIconViewItem::dropped( QDropEvent *e, const QValueList<QIconDragItem> & )
 
 DListView::DListView( QWidget *parent ) : KListView( parent )
 {
+	visibleItems = 0;
 }
 
 
@@ -143,6 +149,9 @@ DvbPanel::DvbPanel( QWidget *parent, QObject *objParent, const char *name ) : Ka
 	configBtn = new QToolButton( panel );
 	configBtn->setAutoRaise( true );
 	QToolTip::add( configBtn, i18n("DVB settings"));
+	recallBtn = new QToolButton( panel );
+	recallBtn->setAutoRaise( true );
+	QToolTip::add( recallBtn, i18n("Recall"));
 	QHBoxLayout *h1 = new QHBoxLayout();
 	h1->addItem( new QSpacerItem( 1, 1, QSizePolicy::Preferred, QSizePolicy::Minimum ) );
 	h1->addWidget( channelsBtn );
@@ -150,35 +159,42 @@ DvbPanel::DvbPanel( QWidget *parent, QObject *objParent, const char *name ) : Ka
 	h1->addWidget( infoBtn );
 	h1->addWidget( osdBtn );
 	h1->addWidget( configBtn );
+	h1->addWidget( recallBtn );
 	h1->addItem( new QSpacerItem( 1, 1, QSizePolicy::Preferred, QSizePolicy::Minimum ) );
 	vb->addLayout( h1 );
 
+	h1 = new QHBoxLayout();
+	searchBtn = new QToolButton( panel );
+	searchBtn->setAutoRaise( true );
+	QToolTip::add( searchBtn, i18n("Search channel(s)"));
+	QLabel* filterLabel = new QLabel( i18n("Filter") + ":", panel );
+	searchLE = new KLineEdit( panel );
+	searchLE->setFocusPolicy( QWidget::ClickFocus );
+	h1->addWidget( searchBtn );
+	h1->addWidget( filterLabel );
+	h1->addWidget( searchLE );
+	vb->addLayout( h1 );
+
 	channelsCb = new DListView( panel );
 	channelsCb->setHScrollBarMode( QListView::AlwaysOff );
 	//channelsCb->setPaletteBackgroundColor( QColor(255,255,200) );
 	connect( channelsCb, SIGNAL(itemRenamed(QListViewItem*)), this, SLOT(channelNumberChanged(QListViewItem*)) );
 	connect( channelsCb, SIGNAL(doubleClicked(QListViewItem*)), this, SLOT(channelSelected(QListViewItem*)) );
+	connect( channelsCb, SIGNAL(clicked(QListViewItem*)), this, SLOT(channelClicked(QListViewItem*)) );
 	connect( channelsCb, SIGNAL(contextMenuRequested(QListViewItem*,const QPoint&,int)), this, SLOT(channelPopup(QListViewItem*,const QPoint&,int)) );
 	channelsCb->setItemsRenameable( true );
 	channelsCb->addColumn( i18n("Number") );
 	channelsCb->addColumn( i18n("Name") );
+	channelsCb->addColumn( i18n("Source") );
 	channelsCb->setAllColumnsShowFocus( true );
 	channelsCb->setSizePolicy( QSizePolicy (QSizePolicy::Preferred, QSizePolicy::MinimumExpanding) );
 	//channelsCb->setEnabled( false );
 	vb->addWidget( channelsCb );
 
 	h1 = new QHBoxLayout();
-	//QLabel *lab = new QLabel( i18n("Audio :"), panel );
-	//lab->resize( lab->sizeHint() );
-	//lab->setSizePolicy( QSizePolicy( QSizePolicy::Fixed, QSizePolicy::Fixed ) );
-	//h1->addWidget( lab );
-	audioComb = new QComboBox( panel );
-	QToolTip::add( audioComb, i18n("Audio Channels") );
-	audioComb->setSizePolicy( QSizePolicy (QSizePolicy::Preferred, QSizePolicy::Fixed) );
-	audioComb->setEnabled( false );
-	h1->addWidget( audioComb );
 	recordBtn = new QToolButton( panel );
 	QToolTip::add( recordBtn, i18n("Instant Record") );
+	recordBtn->setToggleButton( true );
 	h1->addWidget( recordBtn );
 	broadcastBtn = new QToolButton( panel );
 	QToolTip::add( broadcastBtn, i18n("Broadcast") );
@@ -217,20 +233,25 @@ DvbPanel::DvbPanel( QWidget *parent, QObject *objParent, const char *name ) : Ka
 	tvcPix = icon->loadIcon( "kdvbtvc", KIcon::Small );
 	raPix = icon->loadIcon( "kdvbra", KIcon::Small );
 	racPix = icon->loadIcon( "kdvbrac", KIcon::Small );
+	QIconSet iconset;
+	iconset.setPixmap( icon->loadIcon( "filesave", KIcon::Small ), QIconSet::Small );
+	iconset.setPixmap( icon->loadIcon( "player_record", KIcon::Small), QIconSet::Small, QIconSet::Normal, QIconSet::On );
+	recordBtn->setIconSet( iconset );
 	broadcastBtn->setIconSet( icon->loadIconSet("network_local", KIcon::Small) );
-	recordBtn->setIconSet( icon->loadIconSet("filesave", KIcon::Small) );
 	dateBtn->setIconSet( icon->loadIconSet("date", KIcon::Small) );
 	infoBtn->setIconSet( icon->loadIconSet("view_text", KIcon::Small) );
 	osdBtn->setIconSet( icon->loadIconSet("info", KIcon::Small) );
 	channelsBtn->setIconSet( icon->loadIconSet("kdvbtv", KIcon::Small) );
 	configBtn->setIconSet( icon->loadIconSet("configure", KIcon::Small) );
+	recallBtn->setIconSet( icon->loadIconSet("reload", KIcon::Small) );
+	searchBtn->setIconSet( icon->loadIconSet("locationbar_erase", KIcon::Small) );
 
 	setXMLFile("kaffeinedvb.rc");
 	setupActions();
 
-	connect( audioComb, SIGNAL(activated(int)), this, SLOT(audioSelected(int)) );
 	connect( this, SIGNAL(zap(ChannelDesc*)), SLOT(dvbZap(ChannelDesc*)) );
 	connect( configBtn, SIGNAL(clicked()), this, SLOT(showConfigDialog()));
+	connect( recallBtn, SIGNAL(clicked()), this, SLOT(recallZap()));
 
 	connect( recordBtn, SIGNAL(clicked()), this, SLOT(setRecord()) );
 	connect( broadcastBtn, SIGNAL(clicked()), this, SLOT(setBroadcast()) );
@@ -238,18 +259,50 @@ DvbPanel::DvbPanel( QWidget *parent, QObject *objParent, const char *name ) : Ka
 	connect( channelsBtn, SIGNAL(clicked()), this, SLOT(scanner()) );
 	connect( dateBtn, SIGNAL(clicked()), this, SLOT(showTimers()) );
 	connect( osdBtn, SIGNAL(clicked()), this, SLOT(dvbOSD()));
+	connect( searchBtn, SIGNAL(clicked()), this, SLOT(resetSearch()) );
+	connect(searchLE, SIGNAL(textChanged(const QString&)), this, SLOT(searchChannel(const QString&)));
 
 	connect( &timersTimer, SIGNAL(timeout()), this, SLOT(checkTimers()) );
 	connect( &osdTimer, SIGNAL(timeout()), this, SLOT(resetOSD()) );
 	connect( &showOsdTimer, SIGNAL(timeout()), this, SLOT(dvbOSD()) );
 	connect( &tuningTimer, SIGNAL(timeout()), this, SLOT(setTuning()) );
 	connect( &stopTuningTimer, SIGNAL(timeout()), this, SLOT(setTuning()) );
+	connect( &diskTimer, SIGNAL(timeout()), this, SLOT(diskStatus()) );
 
 	setConfig();
 
 	updown = 1;
 	autocount = 0;
-        delete icon;
+	delete icon;
+
+	events.loadEpg();
+}
+
+
+
+void DvbPanel::togglePanel()
+{
+	if ( panel->isHidden() )
+		panel->show();
+	else
+		panel->hide();
+}
+
+
+
+void DvbPanel::diskStatus()
+{
+	double freemb;
+	struct statvfs buf;
+
+	if ( statvfs( dvbConfig->shiftDir.local8Bit(), &buf ) ) {
+		fprintf(stderr,"Couldn't get file system statistics\n");
+		return;
+	}
+
+	freemb = (double)(((double)(buf.f_bavail)*(double)(buf.f_bsize))/(1024.0*1024.0));
+	if ( freemb<300 )
+		emit showOSD( i18n("Warning: low disc space")+QString(" (%1").arg((int)freemb)+i18n("MB")+")", 5000, 3 );
 }
 
 
@@ -269,6 +322,7 @@ void DvbPanel::channelPopup( QListViewItem *it, const QPoint &pos, int col )
 
 	QPopupMenu *pop = new QPopupMenu();
 	pop->insertItem( i18n("Select icon..."), 1 );
+	pop->insertItem( i18n("Edit..."), 2 );
 	i = 0;
 	i = pop->exec( QCursor::pos() );
 	switch ( i ) {
@@ -295,10 +349,58 @@ void DvbPanel::channelPopup( QListViewItem *it, const QPoint &pos, int col )
 			pix.save( dvbConfig->dvbConfigIconsDir+name, "PNG" );
 			fillChannelList();
 			break;
+		case 2:
+			editChannel( name );
+			break;
+
 	}
 	delete pop;
 }
 
+bool DvbPanel::editChannel( QString &name )
+{
+	int j;
+	ChannelDesc *chan=0;
+	QStringList list;
+	QPixmap pix;
+
+	for ( j=0; j<(int)channels.count(); j++ ) {
+		chan = channels.at(j);
+		if ( chan->name==name ) {
+			j = -1;
+			list.append( chan->tp.source );
+			break;
+		}
+	}
+	if ( j==-1 ) {
+		ChannelEditor dlg( list, false, chan, &channels, mainWidget );
+		int ret = dlg.exec();
+		if ( ret==ChannelEditor::Accepted ) {
+			pix.load( dvbConfig->dvbConfigIconsDir+chan->name );
+			if ( !pix.isNull() )
+				chan->pix = pix;
+			else {
+				if ( chan->type==1 ) {
+					if ( chan->fta )
+						pix = tvcPix;
+					else
+						pix = tvPix;
+				}
+				else {
+					if ( chan->fta )
+						pix = racPix;
+					else
+						pix = raPix;
+				}
+			}
+			fillChannelList( chan );
+			return true;
+		}
+	}
+	return false;
+}
+
+
 
 
 void DvbPanel::getTargets( QStringList &uiNames, QStringList &iconNames, QStringList &targetNames )
@@ -328,12 +430,13 @@ bool DvbPanel::execTarget( const QString &target )
 
 void DvbPanel::setupActions()
 {
-	new KAction(i18n("OSD Next Channel"), "next", CTRL + SHIFT + Key_W, this, SLOT(dvbOSDNext()), actionCollection(), "dvb_browse_next");
-	new KAction(i18n("OSD Previous Channel"), "previous", CTRL + SHIFT + Key_Q, this, SLOT(dvbOSDPrev()), actionCollection(),"dvb_browse_prev");
-	new KAction(i18n("OSD Zap"), "ok", CTRL + SHIFT + Key_E, this, SLOT(dvbOSDZap()), actionCollection(), "dvb_browse_zap");
-	new KAction(i18n("OSD Next Event"), "down", CTRL + SHIFT + Key_S, this, SLOT(dvbOSDAdvance()), actionCollection(), "dvb_browse_advance");
-	new KAction(i18n("OSD Previous Event"), "up", CTRL + SHIFT + Key_A, this, SLOT(dvbOSDRetreat()), actionCollection(), "dvb_browse_retreat");
+	new KAction(i18n("OSD Next Channel"), "next", CTRL+SHIFT+Key_W, this, SLOT(dvbOSDNext()), actionCollection(), "dvb_browse_next");
+	new KAction(i18n("OSD Previous Channel"), "previous", CTRL+SHIFT+Key_Q, this, SLOT(dvbOSDPrev()), actionCollection(),"dvb_browse_prev");
+	new KAction(i18n("OSD Zap"), "ok", CTRL+SHIFT+Key_E, this, SLOT(dvbOSDZap()), actionCollection(), "dvb_browse_zap");
+	new KAction(i18n("OSD Next Event"), "down", CTRL+SHIFT+Key_S, this, SLOT(dvbOSDAdvance()), actionCollection(), "dvb_browse_advance");
+	new KAction(i18n("OSD Previous Event"), "up", CTRL+SHIFT+Key_A, this, SLOT(dvbOSDRetreat()), actionCollection(), "dvb_browse_retreat");
 	new KAction(i18n("Instant Record"), "filesave", 0, this, SLOT(setRecord()), actionCollection(), "dvb_instant_record");
+	new KAction(i18n("Recall"), "reload", CTRL+SHIFT+Key_Z, this, SLOT(recallZap()), actionCollection(), "dvb_recall");
 	new KAction(i18n("Show OSD"), "info", Key_O, this, SLOT(dvbOSD()), actionCollection(), "dvb_show_osd");
 	new KAction(i18n("EPG..."), "view_text", Key_G, this, SLOT(showEvents()), actionCollection(), "dvb_show_epg");
 	new KAction(i18n("Timers..."), "date", Key_T, this, SLOT(showTimers()), actionCollection(), "dvb_show_timers");
@@ -358,6 +461,45 @@ QWidget *DvbPanel::inputMainWidget()
 
 
 
+void DvbPanel::searchChannel( const QString &text )
+{
+	int i, c=-1;
+	QListViewItemIterator it( channelsCb );
+	if ( !it.current() )
+		return;
+	for ( i=0; i<channelsCb->columns(); i++ ) {
+		if ( channelsCb->columnText(i)==i18n("Name") ) {
+			c = i;
+			break;
+		}
+	}
+	if ( c==-1 )
+		return;
+
+	channelsCb->visibleItems = 0;
+
+	while ( it.current() ) {
+		if ( text.isEmpty() || it.current()->text( c ).contains( text, false ) )	{
+			it.current()->setVisible(true);
+			++channelsCb->visibleItems;
+		}
+		else {
+			it.current()->setVisible(false);
+		}
+		++it;
+	}
+}
+
+
+
+void DvbPanel::resetSearch()
+{
+	searchLE->clear();
+	searchChannel( "" );
+}
+
+
+
 void DvbPanel::channelNumberChanged( QListViewItem *it )
 {
 	int i, j;
@@ -485,10 +627,11 @@ void DvbPanel::moveChannel( const QString &cat, const QString &name )
 
 void DvbPanel::dumpEvents()
 {
-	DVBevents *events=0;
-	EventSourceList *slist;
-	EventTsidList *tlist;
-	EventDesc *desc=0;
+	EventSource *esrc;
+	EventTsid *et;
+	EventSid *es;
+	QPtrList<EventDesc> *esEvents;
+	EventDesc *desc;
 	int i, k, j, m, n;
 	QString s;
 
@@ -498,15 +641,19 @@ void DvbPanel::dumpEvents()
 		QTextStream tt( &f );
 		tt << "Saved: "+QDateTime::currentDateTime().toString( "dd-MM-yyyy hh:mm:ss" )+"\n";
 		k= 0;
-		for( i=0; i<(int)dvb.count(); i++ ) {
-			events = dvb.at(i)->dvbEvents;
-			for ( m=0; m<(int)events->srcList.count(); m++ ) {
-				slist = events->srcList.at(m);
-				tt << slist->source+" : "+s.setNum( slist->tsidList.count() )+" TS with events.\n";
-				for ( n=0; n<(int)slist->tsidList.count(); n++ ) {
-					tlist = slist->tsidList.at(n);
-					tt << "TSID "+s.setNum( tlist->tsid )+" : "+s.setNum( tlist->events.count() )+" events\n";
-					k+= tlist->events.count();
+		for( i=0; i<events.getNSource(); i++ ) {
+			if ( !(esrc=events.getNEventSource( i )) )
+				continue;
+			tt << esrc->getSource()+QString(" : %1 TS with events.\n").arg( esrc->getNTsid() );
+			for ( m=0; m<esrc->getNTsid(); m++ ) {
+				if ( !(et=esrc->getNEventTsid( m )) )
+					continue;
+				tt << QString("TSID %1 : %2 services with events\n").arg( et->getTsid() ).arg( et->getNSid() );
+				for ( n=0; n<et->getNSid(); n++ ) {
+					if ( !(es=et->getNEventSid( n )) )
+						continue;
+					tt << QString("     SID %1 : %2 events\n").arg( es->getSid() ).arg( es->getNDesc() );
+					k+= es->getNDesc();
 				}
 			}
 		}
@@ -514,20 +661,25 @@ void DvbPanel::dumpEvents()
 		fprintf( stderr, "Number of events : %d\n", k );
 		tt << "-------------------------\n";
 		tt << "\n";
-		for( i=0; i<(int)dvb.count(); i++ ) {
-			events = dvb.at(i)->dvbEvents;
-			events->mutex.lock();
-			for ( m=0; m<(int)events->srcList.count(); m++ ) {
-				slist = events->srcList.at(m);
-				for ( n=0; n<(int)slist->tsidList.count(); n++ ) {
-					tlist = slist->tsidList.at(n);
-					for ( j=0; j<(int)tlist->events.count(); j++ ) {
-						desc = tlist->events.at(j);
+		for( i=0; i<events.getNSource(); i++ ) {
+			if ( !(esrc=events.getNEventSource( i )) )
+				continue;
+			for ( m=0; m<esrc->getNTsid(); m++ ) {
+				if ( !(et=esrc->getNEventTsid( m )) )
+					continue;
+				for ( n=0; n<et->getNSid(); n++ ) {
+					if ( !(es=et->getNEventSid( n )) )
+						continue;
+					esEvents = es->getEvents();
+					es->lock();
+					for ( j=0; j<(int)esEvents->count(); j++ ) {
+						if ( !(desc=esEvents->at( j )) )
+							continue;
 						tt << "Source: "+desc->source+"\n";
-						tt << "Network ID: "+s.setNum(desc->nid)+"\n";
-						tt << "Transport Stream ID: "+s.setNum(desc->tsid)+"\n";
-						tt << "Service ID: "+s.setNum(desc->sid)+"\n";
-						tt << "Event ID: "+s.setNum(desc->eid)+"\n";
+						tt << QString("Network ID: %1\n").arg(desc->nid);
+						tt << QString("Transport Stream ID: %1\n").arg(desc->tsid);
+						tt << QString("Service ID: %1\n").arg(desc->sid);
+						tt << QString("Event ID: %1\n").arg(desc->eid);
 						tt << "Title: "+desc->title+"\n";
 						tt << "Subtitle: "+desc->subtitle+"\n";
 						tt << desc->startDateTime.toString( "dd-MM-yyyy hh:mm:ss" )+"\n";
@@ -546,9 +698,9 @@ void DvbPanel::dumpEvents()
 						tt << s+"\n";
 						tt << "\n";
 					}
+					es->unlock();
 				}
 			}
-			events->mutex.unlock();
 		}
 		f.close();
 	}
@@ -557,6 +709,40 @@ void DvbPanel::dumpEvents()
 
 
 
+void DvbPanel::channelClicked( QListViewItem *it )
+{
+	QPtrListIterator<ChannelDesc> iter( channels );
+	ChannelDesc *itdesc, *desc=0;
+	DvbStream *d = 0;
+
+	if ( !it )
+		return;
+
+	iter.toFirst();
+	while ( (itdesc=iter.current())!=0 ) {
+		if ( itdesc->name==it->text(1) ) {
+			desc = itdesc;
+			break;
+		}
+		++iter;
+	}
+
+	if ( !desc )
+		return;
+
+	for (int j = 0; j < (int)dvb.count(); j ++) {
+		if (dvb.at(j)->canSource(desc)) {
+			d = dvb.at(j);
+			break;
+		}
+	}
+
+	osdMode = 0;
+	dvbOSD( *desc, d );
+}
+
+
+
 void DvbPanel::resetOSD()
 {
 	osdMode = 0;
@@ -703,19 +889,15 @@ void DvbPanel::dvbOSDRetreat()
 void DvbPanel::dvbOSD(ChannelDesc liveChannel, DvbStream *d, int timeShift /* = 0 */)
 {
 	QStringList list;
-	DVBevents *events;
+	EventSource *esrc;
+	EventSid *es;
+	QPtrList<EventDesc> *esEvents;
 	EventDesc *desc;
-	EventSourceList *slist=0;
-	EventTsidList *tlist=0;
-	QPtrList<EventDesc> *currentEvents=0;
 	QString s;
-	int i=0, j, k;
+	int i, j, k;
 	int first=0;
 
-	if ( !d )
-		return;
-
-	if ( liveChannel.name.isEmpty() )
+	if ( !d || liveChannel.name.isEmpty() )
 		return;
 
 	if ( osdMode==2 ) {
@@ -726,8 +908,6 @@ void DvbPanel::dvbOSD(ChannelDesc liveChannel, DvbStream *d, int timeShift /* =
 		return;
 	}
 
-	events = d->dvbEvents;
-
 	if ( osdMode==1 )
 		osdTimer.stop();
 
@@ -753,42 +933,18 @@ void DvbPanel::dvbOSD(ChannelDesc liveChannel, DvbStream *d, int timeShift /* =
 	browseDvbTimeShift = myshift;
 
 	k = 0;
-	int myShifted = 0;
 
-	events->mutex.lock();
-	for ( j=0; j<(int)events->srcList.count(); j++ ) {
-		if ( events->srcList.at(j)->source==liveChannel.tp.source ) {
-			slist = events->srcList.at(i);
-			break;
-		}
-	}
-	if ( !slist ) {
+	esrc = events.getEventSource( liveChannel.tp.source );
+	es = esrc->getEventSid( liveChannel.tp.nid, liveChannel.tp.tsid, liveChannel.sid );
+	if ( !es )
 		goto end;
-	}
+	es->lock();
+	esEvents = es->getEvents();
 
-	for ( j=0; j<(int)slist->tsidList.count(); j++ ) {
-		tlist = slist->tsidList.at(j);
-		if ( tlist->tsid==liveChannel.tp.tsid ) {
-			currentEvents = &tlist->events;
-			break;
-		}
-	}
-	if ( !currentEvents ) {
-		goto end;
-	}
-
-	for ( j=0; j<(int)currentEvents->count(); j++ ) {
-		desc = currentEvents->at(j);
-
-		//if ( desc->tid>0x4f ) continue;
-		if ( desc->source!=liveChannel.tp.source || desc->sid!=liveChannel.sid || desc->tsid!=liveChannel.tp.tsid )
+	for ( j=myshift; j<(myshift+2); j++ ) {
+		desc = esEvents->at( j );
+		if ( !desc )
 			continue;
-		//if ( desc->running!=4 && desc->running!=1 ) continue;
-
-		if (myShifted < myshift) {
-			myShifted++;
-			continue;
-		}
 
 		if ( !desc->title.isEmpty() ) {
 			s = desc->startDateTime.toString( "hh:mm" );
@@ -831,8 +987,9 @@ void DvbPanel::dvbOSD(ChannelDesc liveChannel, DvbStream *d, int timeShift /* =
 		list.append( s );
 		break;
 	}
+	es->unlock();
+
 end:
-	events->mutex.unlock();
 	if ( osdMode==0 )
 		osdTimer.start( 5000, true );
 	osdMode++;
@@ -880,7 +1037,7 @@ void DvbPanel::setConfig()
 		plugName = ptr->desktopEntryName();
 		plug = KParts::ComponentFactory::createPartInstanceFromService<KaffeineDvbPlugin>(ptr, 0, ptr->name().ascii(), 0, 0, 0, &error);
 		if (error > 0) {
-			fprintf( stderr, "Loading of DVB plugin failed\n.");
+			fprintf( stderr, "Loading of DVB plugin failed: %s\n", KLibLoader::self()->lastErrorMessage().ascii() );
 			plug = NULL;
 		}
 		else {
@@ -894,11 +1051,7 @@ void DvbPanel::setConfig()
 	}
 
 	for ( i=0; i<(int)dvbConfig->devList.count(); i++ ) {
-		d = new DvbStream( dvbConfig->devList.at(i)->adapter, dvbConfig->devList.at(i)->tuner, dvbConfig->devList.at(i)->type, dvbConfig->devList.at(i)->source, dvbConfig->devList.at(i)->canAutoscan, dvbConfig->defaultCharset );
-		d->setLnb( 0, dvbConfig->devList.at(i)->lnb0 );
-		d->setLnb( 1, dvbConfig->devList.at(i)->lnb1 );
-		d->setLnb( 2, dvbConfig->devList.at(i)->lnb2 );
-		d->setLnb( 3, dvbConfig->devList.at(i)->lnb3 );
+		d = new DvbStream( dvbConfig->devList.at(i), dvbConfig->defaultCharset, &events );
 		d->setPlug( plug );
 		dvb.append( d );
 		connect( d, SIGNAL(shifting(bool)), this, SLOT(setShiftLed(bool)) );
@@ -929,15 +1082,16 @@ void DvbPanel::setConfig()
 	getChannelList();
 	rtp = new Ts2Rtp();
 	rtp->setSocket( dvbConfig->broadcastAddress, dvbConfig->broadcastPort, dvbConfig->senderPort );
-	cleaner = new Cleaner( dvbConfig->shiftDir );
+	cleaner = new Cleaner( dvbConfig->shiftDir, dvbConfig->recordDir );
 	split->setSizes( dvbConfig->splitSizes );
+	recallChannel = dvbConfig->lastChannel;
 }
 
 
 
 void DvbPanel::showConfigDialog()
 {
-	int ret, i;
+	int ret;
 
 loop:
 	if ( !dvbConfig->haveData() ) {
@@ -956,15 +1110,8 @@ loop:
 	disconnect( dlg.dumpBtn, SIGNAL(clicked()), this, SLOT(dumpEvents()) );
 	if ( ret==DvbConfigDialog::Rejected )
 		return;
-	for ( i=0; i<(int)dvbConfig->devList.count(); i++ ) {
-		dvb.at(i)->setSources( dvbConfig->devList.at(i)->source );
-		dvb.at(i)->setLnb( 0, dvbConfig->devList.at(i)->lnb0 );
-		dvb.at(i)->setLnb( 1, dvbConfig->devList.at(i)->lnb1 );
-		dvb.at(i)->setLnb( 2, dvbConfig->devList.at(i)->lnb2 );
-		dvb.at(i)->setLnb( 3, dvbConfig->devList.at(i)->lnb3 );
-	}
 	rtp->setSocket( dvbConfig->broadcastAddress, dvbConfig->broadcastPort, dvbConfig->senderPort );
-	cleaner->setPath( dvbConfig->shiftDir );
+	cleaner->setPaths( dvbConfig->shiftDir, dvbConfig->recordDir );
 }
 
 
@@ -993,15 +1140,16 @@ QPtrList<Transponder> DvbPanel::getSourcesStatus()
 
 
 
-void DvbPanel::fillChannelList()
+void DvbPanel::fillChannelList( ChannelDesc *ch )
 {
 	int i, j;
 	ChannelDesc *chan;
-	KListViewItem *it;
+	KListViewItem *it, *visible=0;
 	bool cont=false;
 	QPtrList<Transponder> trans = getSourcesStatus();
 	trans.setAutoDelete( true );
 
+	searchLE->clear();
 	channelsCb->clear();
 	for ( i=0; i<(int)channels.count(); i++ ) {
 		chan = channels.at(i);
@@ -1028,8 +1176,11 @@ void DvbPanel::fillChannelList()
 		}
 		else if ( currentCategory!="All" && chan->category!=currentCategory )
 			continue;
-		it = new KListViewItem( channelsCb, QString().sprintf("%04d", chan->num), chan->name );
+		it = new KListViewItem( channelsCb, QString().sprintf("%04d", chan->num), chan->name, chan->tp.source );
+		if ( ch && ch==chan )
+			visible = it;
 		it->setDragEnabled( true );
+		chan->pix.load( dvbConfig->dvbConfigIconsDir+chan->name );
 		if ( !chan->pix.isNull() )
 			it->setPixmap( 1, chan->pix );
 		else {
@@ -1047,7 +1198,12 @@ void DvbPanel::fillChannelList()
 			}
 		}
 	}
+	channelsCb->visibleItems = channelsCb->childCount();
 	trans.clear();
+	if ( visible ) {
+		channelsCb->ensureItemVisible( visible );
+		channelsCb->setSelected( visible, true );
+	}
 }
 
 
@@ -1171,7 +1327,7 @@ void DvbPanel::checkTimers()
 					stopLive();
 					emit dvbStop();
 				}
-				if ( d->startTimer( chan, dvbConfig->recordDir, t ) ) {
+				if ( d->startTimer( chan, dvbConfig->recordDir, dvbConfig->sizeFile, t, dvbConfig->filenameFormat ) ) {
 					KProcess proc;
 					proc << QDir::homeDirPath()+"/bin/kaffeine_recording";
 					proc << "On";
@@ -1180,7 +1336,6 @@ void DvbPanel::checkTimers()
 					if ( timersDialog )
 						emit timersChanged();
 					saveTimerList();
-					audioComb->setEnabled( false );
 					i--;
 				}
 				else
@@ -1205,13 +1360,10 @@ void DvbPanel::setRecord()
 {
 	ChannelDesc curchan;
 	QString s="";
-	int ret, i, j, r=0;
+	int ret, i, r=0;
 	bool live=false;
 	RecTimer *t, *rt;
 	EventDesc *desc;
-	EventSourceList *slist=0;
-	EventTsidList *tlist=0;
-	QPtrList<EventDesc> *currentEvents=0;
 	DvbStream *d=0;
 
 	for ( i=0; i<(int)dvb.count(); i++ ) {
@@ -1222,58 +1374,39 @@ void DvbPanel::setRecord()
 		}
 	}
 	if ( !d ) {
+		recordBtn->setOn( false );
 		showTimers();
 		return;
 	}
 
-	d->dvbEvents->mutex.lock();
-	for ( j=0; j<(int)d->dvbEvents->srcList.count(); j++ ) {
-		if ( d->dvbEvents->srcList.at(j)->source==curchan.tp.source ) {
-			slist = d->dvbEvents->srcList.at(j);
-			break;
-		}
-	}
-	if ( slist ) {
-		for ( j=0; j<(int)slist->tsidList.count(); j++ ) {
-			tlist = slist->tsidList.at(j);
-			if ( tlist->tsid==curchan.tp.tsid ) {
-				currentEvents = &tlist->events;
-				break;
-			}
-		}
-		if ( currentEvents ) {
-			for ( j=0; j<(int)currentEvents->count(); j++ ) {
-				desc = currentEvents->at(j);
-				if ( desc->sid!=curchan.sid || desc->tsid!=curchan.tp.tsid )
-					continue;
-				if ( desc->running!=4 )
-					continue;
-				s = desc->title;
-				break;
-			}
-		}
-	}
-	d->dvbEvents->mutex.unlock();
+	desc = events.getEventDesc( curchan.tp.source, curchan.tp.nid, curchan.tp.tsid, curchan.sid, 0 );
 
-	if ( s.isEmpty() )
-		s = curchan.name+"-"+QDateTime::currentDateTime().toString("yyyyMMdd-hhmmss");
+	if ( desc )
+		s = desc->title;
+
+	if ( s.isEmpty() ) {
+		if ( !dvbConfig->filenameFormat.contains("%chan") )
+			s = curchan.name;
+		if ( !dvbConfig->filenameFormat.contains("%date") )
+			s+="_"+QDateTime::currentDateTime().toString( "yyyyMMdd-hhmmss" );
+	}
 
 	rt = new RecTimer();
-	rt->name = s.replace( "/", "_" ).replace( ">", "_" ).replace("<","_").replace(":","_").replace('"',"_").replace("\\","_").replace("|","_");
+	rt->name = s;
 	rt->channel = curchan.name;
 	rt->begin = QDateTime::currentDateTime();
 	rt->duration = QTime( 0,0,0).addSecs( dvbConfig->instantDuration*60 ) ;
-	rt->filetype = dvbConfig->format;
 	rt->running = 1;
 	rt->mode = 0;
 
 	ret = d->canStartTimer( live, &curchan );
 	if ( ret==0 ) {
-		if ( d->startTimer( &curchan, dvbConfig->recordDir, rt ) ) {
+		if ( d->startTimer( &curchan, dvbConfig->recordDir, dvbConfig->sizeFile, rt, dvbConfig->filenameFormat ) ) {
 			KProcess proc;
 			proc << QDir::homeDirPath()+"/bin/kaffeine_recording";
 			proc << "On";
 			proc.start( KProcess::DontCare );
+
 			for ( i=0; i<(int)timers.count(); i++ ) {
 				t = timers.at(i);
 				if ( rt->begin>t->begin )
@@ -1283,16 +1416,24 @@ void DvbPanel::setRecord()
 			if ( timersDialog )
 				emit timersChanged();
 			saveTimerList();
-			audioComb->setEnabled( false );
-			KMessageBox::information( mainWidget, i18n("Timer successfully created and started.") );
+			emit showOSD( i18n("Instant Record successfully started"), 5000, 3 );
 		}
 		else {
-			fprintf( stderr, "start timer failed!\n" );
+			KMessageBox::information( mainWidget, i18n("Instant Recording failed to start.") );
+			fprintf( stderr, "DvbPanel::setRecord:start timer failed!\n" );
 			delete rt;
 		}
 	}
 	else {
 		delete rt;
+		for ( i=0; i<(int)timers.count(); i++ ) {
+			t = timers.at(i);
+			if ( d->liveIsRecording() && t->running && curchan.name == t->channel ) {
+				d->updateTimer( t, 0 );
+				emit showOSD( i18n("Recording successfully stopped"), 5000, 3 );
+				return;
+			}
+		}
 		showTimers();
 	}
 }
@@ -1321,6 +1462,14 @@ void DvbPanel::setRecordLed( bool on )
 			recordLed->on();
 		fillChannelList();
 	}
+
+	for ( i=0; i<(int)dvb.count(); i++ ) {
+		if ( dvb.at(i)->liveIsRecording() ) {
+			recordBtn->setOn( true );
+			return;
+		}
+	}
+	recordBtn->setOn( false );
 }
 
 
@@ -1364,6 +1513,7 @@ void DvbPanel::killTimer( RecTimer *t )
 				proc << "Off";
 			proc << KProcess::quote( t->fullPath );
 			proc.start( KProcess::DontCare );
+
 			if ( t->mode )
 				updateModeTimer( t );
 			else
@@ -1403,7 +1553,6 @@ void DvbPanel::updateModeTimer( RecTimer *t )
 	nt->channel = t->channel;
 	nt->begin = t->begin;
 	nt->duration = t->duration;
-	nt->filetype = t->filetype;
 	nt->running = 0;
 	nt->mode = t->mode;
 	timers.remove( t );
@@ -1434,7 +1583,7 @@ void DvbPanel::showTimers()
         for ( it = map.begin(); it != end; ++it )
         	list.append( it.data() );
 
-	timersDialog = new KRecord( list, &timers, mainWidget, dvbConfig->timerSize, dvbConfig->format );
+	timersDialog = new KRecord( list, &timers, mainWidget, dvbConfig->timerSize );
 	for ( i=0; i<(int)dvb.count(); i++ ) {
 		connect( timersDialog, SIGNAL(updateTimer(RecTimer*,int)), dvb.at(i), SLOT(updateTimer(RecTimer*,int)) );
 	}
@@ -1452,6 +1601,14 @@ void DvbPanel::showTimers()
 
 
 
+int DvbPanel::getSNR( int device )
+{
+	if ( device<0 || device>=(int)dvb.count() )
+		return -1;
+	return dvb.at(device)->getSNR();
+}
+
+
 void DvbPanel::dvbNewTimer( QString name, QString channel, QString datetime, QString duration )
 {
 	newTimer( channel, name, QDateTime::fromString( datetime, Qt::ISODate ), QTime::fromString( duration ), false );
@@ -1469,7 +1626,6 @@ void DvbPanel::newTimer( QString channel, QString name, QDateTime begin, QTime d
 	rt->channel = channel;
 	rt->begin = begin.addSecs( -(dvbConfig->beginMargin*60) );
 	rt->duration = duration.addSecs( (dvbConfig->beginMargin+dvbConfig->endMargin)*60 ) ;
-	rt->filetype = dvbConfig->format;
 	rt->running = 0;
 	rt->mode = 0;
 
@@ -1513,19 +1669,15 @@ loop:
 
 void DvbPanel::showEvents()
 {
-	int i;
-
-	for ( i=0; i<(int)dvb.count(); i++ )
-		dvb.at(i)->dvbEvents->doClean( false );
-	KEvents dlg( &channels, &dvb, mainWidget, dvbConfig->epgSize );
+	events.doClean( false );
+	KEvents dlg( &channels, &dvb, &events, mainWidget, dvbConfig->epgSize );
 	connect( &dlg, SIGNAL(addTimer(QString,QString,QDateTime,QTime)), this, SLOT(newTimer(QString,QString,QDateTime,QTime)) );
 	connect( &dlg, SIGNAL(zapTo(const QString &)), this, SLOT(channelSelected(const QString &)) );
 	dlg.exec();
 	dvbConfig->epgSize = dlg.size();
 	disconnect( &dlg, SIGNAL(addTimer(QString,QString,QDateTime,QTime)), this, SLOT(newTimer(QString,QString,QDateTime,QTime)) );
 	disconnect( &dlg, SIGNAL(zapTo(const QString &)), this, SLOT(channelSelected(const QString &)) );
-	for ( i=0; i<(int)dvb.count(); i++ )
-		dvb.at(i)->dvbEvents->doClean( true );
+	events.doClean( true );
 }
 
 
@@ -1534,10 +1686,12 @@ void DvbPanel::setShiftLed( bool on )
 {
 	if ( on ) {
 		shiftLed->on();
-		audioComb->setEnabled( false );
+		diskTimer.start( 300000 );
 	}
-	else
+	else {
 		shiftLed->off();
+		diskTimer.stop();
+	}
 }
 
 
@@ -1579,30 +1733,6 @@ void DvbPanel::channelSelected( QListViewItem *it )
 
 
 
-void DvbPanel::audioSelected( int n )
-{
-	int i;
-	DvbStream *d=0;
-
-	for ( i=0; i<(int)dvb.count(); i++ ) {
-		if ( dvb.at(i)->hasLive() ) {
-			d = dvb.at(i);
-			break;
-		}
-	}
-	if ( !d )
-		return;
-
-	ChannelDesc chan=d->getLiveChannel();
-	emit setTimeShiftFilename( "" );
-	d->preStopLive();
-	emit dvbPause( false );
-	d->stopLive( &chan );
-	finalZap( d, &chan, false, n );
-}
-
-
-
 bool DvbPanel::nextTrack( MRL& )
 {
 	next();
@@ -1635,6 +1765,14 @@ bool DvbPanel::trackNumber( int num, MRL& )
 
 
 
+void DvbPanel::recallZap()
+{
+	emit showMe( this );
+	playNumChannel( recallChannel );
+}
+
+
+
 void DvbPanel::playNumChannel( int num )
 {
 	int j;
@@ -1721,20 +1859,14 @@ void DvbPanel::playLastChannel()
 	if ( !channelsCb->isEnabled() )
 		return;
 
-	//++dvbConfig->lastChannel;
-	//if ( dvbConfig->lastChannel>channels.count() )
-	//	dvbConfig->lastChannel = 1;
-
 	for ( j=0; j<(int)channels.count(); j++ ) {
 		if ( (int)channels.at(j)->num==dvbConfig->lastChannel ) {
 			ok = true;
 			break;
 		}
 	}
-	if ( !ok ) {
-		//QTimer::singleShot( 3000, this, SLOT(playLastChannel()) );
+	if ( !ok )
 		return;
-	}
 
 	dvbZap( channels.at(j) );
 }
@@ -1801,19 +1933,22 @@ void DvbPanel::dvbZap( ChannelDesc *chan )
 		return;
 
 	isTuning = true;
-	audioComb->setEnabled( false );
 	emit setTimeShiftFilename( "" );
+	for ( i=0; i<(int)dvb.count(); i++ ) {
+		if ( dvb.at(i)->getCurrentTransponder()==chan->tp ) {
+			d = dvb.at(i);
+			break;
+		}
+	}
 	for ( i=0; i<(int)dvb.count(); i++ ) {
 		if ( dvb.at(i)->hasLive() ) {
-			if ( dvb.at(i)->getLiveChannel().tp==chan->tp )
-				d = dvb.at(i);
 			dvb.at(i)->preStopLive();
 			emit dvbPause( false );
 			dvb.at(i)->stopLive( chan );
 			break;
 		}
 	}
-	finalZap( d, chan, true );
+	finalZap( d, chan );
 }
 
 
@@ -1834,10 +1969,9 @@ void DvbPanel::setTuning()
 
 
 
-void DvbPanel::finalZap( DvbStream *d, ChannelDesc *chan, bool setAudio, int napid )
+void DvbPanel::finalZap( DvbStream *d, ChannelDesc *chan )
 {
 	QString s, t;
-	int apidn=napid;
 	int i;
 	DvbStream *d1=d, *d2=0;
 
@@ -1872,7 +2006,7 @@ void DvbPanel::finalZap( DvbStream *d, ChannelDesc *chan, bool setAudio, int nap
 		return;
 	}
 
-	int ret = d1->goLive( chan, currentFifo, apidn );
+	int ret = d1->goLive( chan, currentFifo );
 
 	switch ( ret ) {
 		case DvbStream::ErrIsRecording :
@@ -1892,21 +2026,6 @@ void DvbPanel::finalZap( DvbStream *d, ChannelDesc *chan, bool setAudio, int nap
 			break;
 	}
 
-	if ( setAudio ) {
-		audioComb->clear();
-		for ( i=0; i<chan->napid; i++ ) {
-			s = t.setNum( chan->apid[i].pid );
-			if ( !chan->apid[i].lang.isEmpty() )
-				s = s+"("+chan->apid[i].lang+")";
-			if ( chan->apid[i].ac3 )
-				s = s+"(ac3)";
-			audioComb->insertItem( s );
-		}
-		audioComb->setCurrentItem( apidn );
-		if ( chan->napid>1 && ret==0 )
-			audioComb->setEnabled( true );
-	}
-
 	fprintf( stderr, "Tuning delay: %d ms\n", tm.elapsed() );
 	osdMode = 0;
 	if ( ret<1 ) {
@@ -1922,12 +2041,24 @@ void DvbPanel::finalZap( DvbStream *d, ChannelDesc *chan, bool setAudio, int nap
 			if ( !fifoName.isEmpty() )
 				currentFifo = fifoName;
 		}
+		if ( channelsCb->visibleItems<5 ) {
+			resetSearch();
+		}
+
+		if ( d1->liveIsRecording() )
+			recordBtn->setOn( true );
+		else
+			recordBtn->setOn( false );
 	}
 	else {
+		recordBtn->setOn( false );
 		isTuning = false;
 		emit dvbStop();
 	}
-	dvbConfig->lastChannel = chan->num;
+	if ( dvbConfig->lastChannel!=(int)chan->num ) {
+		recallChannel = dvbConfig->lastChannel;
+		dvbConfig->lastChannel = chan->num;
+	}
 	//QTimer::singleShot( 3000, this, SLOT(playLastChannel()) );
 }
 
@@ -1976,18 +2107,18 @@ void DvbPanel::stopLive()
 
 	for ( i=0; i<(int)dvb.count(); i++ ) {
 		if ( dvb.at(i)->hasLive() ) {
+			recallChannel = dvb.at(i)->getLiveChannel().num;
 			dvb.at(i)->preStopLive();
 			dvb.at(i)->stopLive( &chan );
 			break;
 		}
 	}
+	recordBtn->setOn( false );
 	emit setTimeShiftFilename( "" );
-	audioComb->clear();
-	audioComb->setEnabled( false );
 }
 
 
-//   TV | vpid(stream_type) | apid1(lang)(ac3),apid2, | ttpid | sid | tsid | type(S,C,T)source | freq | sr | pol | fecH | inv | mod | fecL | bw | trans | guard | hier | number / subpid1(type)(pageid)(ancid)(lang),subpid2..., | category |
+//   TV | vpid(stream_type) | apid1(lang)(ac3),apid2, | ttpid | sid | tsid | type(S,C,T)source | freq | sr | pol | fecH | inv | mod | fecL | bw | trans | guard | hier | number / subpid1(type)(pageid)(ancid)(lang),subpid2..., | category | nid |
 bool DvbPanel::getChannelList()
 {
 	bool ret=false;
@@ -2002,6 +2133,20 @@ bool DvbPanel::getChannelList()
 	maxChannelNumber = 0;
 	minChannelNumber = -1;
 
+	int sort = dvbConfig->readDvbChanOrder();
+	if ( sort ) {
+		int column = sort&1;
+		int order = sort>>1;
+		switch (order){
+			case 1:
+				channelsCb->setSorting ( column, FALSE );
+				break;
+			case 0:
+				channelsCb->setSorting ( column, TRUE );
+				break;
+		}
+	}
+
 	QFile f( locateLocal("appdata", "channels.dvb" ) );
 	if ( f.open(IO_ReadOnly) ) {
 		QTextStream tt( &f );
@@ -2208,6 +2353,9 @@ bool DvbPanel::getChannelList()
 			}
 			pos = s.find("|");
 			chan->category = s.left( pos );
+			s = s.right( s.length()-pos-1 );
+			pos = s.find("|");
+			chan->tp.nid = s.left(pos).toUShort();
 
 			if ( chan->tp.source.isEmpty() ) {
 				delete chan;
@@ -2215,7 +2363,7 @@ bool DvbPanel::getChannelList()
 			}
 
 			chan->pix.load( dvbConfig->dvbConfigIconsDir+chan->name );
-			it = new KListViewItem( channelsCb, QString().sprintf("%04d", chan->num), chan->name );
+			it = new KListViewItem( channelsCb, QString().sprintf("%04d", chan->num), chan->name, chan->tp.source );
 			it->setDragEnabled( true );
 			if ( !chan->pix.isNull() )
 				it->setPixmap( 1, chan->pix );
@@ -2384,7 +2532,8 @@ bool DvbPanel::saveChannelList()
 				tt<< ",";
 			}
 			tt<< "|";
-			tt<< chan->category+"|\n";
+			tt<< chan->category+"|";
+			tt<< s.setNum(chan->tp.nid)+"|\n";
 		}
 		ret = true;
 		f.close();
@@ -2422,7 +2571,7 @@ bool DvbPanel::getTimerList()
 			t->duration = QTime::fromString( s.left( pos ) );
 			s = s.right( s.length()-pos-1 );
 			pos = s.find("|");
-			t->filetype = s.left( pos ).toInt();
+			//t->filetype = s.left( pos ).toInt();
 			t->mode = 0;
 			s = s.right( s.length()-pos-1 );
 			pos = s.find("|");
@@ -2457,7 +2606,7 @@ bool DvbPanel::saveTimerList()
 			tt<< t->channel+"|";
 			tt<< t->begin.toString("yyyy-MM-ddThh:mm:ss")+"|";
 			tt<< t->duration.toString()+"|";
-			tt<< s.setNum(t->filetype)+"|";
+			tt<< s.setNum(0/*t->filetype*/)+"|";
 			tt<< s.setNum(t->mode)+"|";
 			tt<< "\n";
 		}
@@ -2503,7 +2652,9 @@ void DvbPanel::saveConfig()
 	saveTimerList();
 	saveChannelList();
 	dvbConfig->splitSizes = split->sizes();
-	delete dvbConfig;
+	dvbConfig->saveDvbChanOrder( channelsCb->sortOrder(), channelsCb->sortColumn() );
+	dvbConfig->saveConfig();
+	//delete dvbConfig;
 }
 
 
@@ -2516,6 +2667,7 @@ DvbPanel::~DvbPanel()
 		KService::Ptr service = KService::serviceByDesktopName( plugName );
 		KLibLoader::self()->unloadLibrary( service->library().ascii() );
 	}
+	events.saveEpg();
 }
 
 #include "dvbpanel.moc"
