@@ -24,9 +24,6 @@
 
 #include <qdir.h>
 
-#include <kmessagebox.h>
-#include <klocale.h>
-#include <kprocess.h>
 #include <kapplication.h>
 
 #include "dvbsi.h"
@@ -35,12 +32,10 @@
 
 
 
-NitSection::NitSection( QPtrList<Transponder> *tp, bool *end, bool *ok, int anum, int tnum ) : DVBsection( ok, anum, tnum )
+NitSection::NitSection( QPtrList<Transponder> *tp, bool *end, bool *ok, int anum, int tnum ) : KaffeineDVBsection( anum, tnum )
 {
 	ended = end;
 	transponders = tp;
-	pf[0].fd = fdDemux;
-	pf[0].events = POLLIN;
 	start();
 }
 
@@ -55,7 +50,7 @@ NitSection::~NitSection()
 
 void NitSection::stop()
 {
-	if ( !wait(1000) ) {
+	if ( !wait(10000) ) {
 		terminate();
 		wait();
 	}
@@ -337,7 +332,7 @@ void NitSection::freqListDesc( unsigned char* buf, Transponder *trans )
 
 
 
-DVBsi::DVBsi( bool *ok, int anum, int tnum, DvbStream *d, const QString &charset ) : DVBsection( ok, anum, tnum, charset )
+DVBsi::DVBsi( bool *ok, int anum, int tnum, DvbStream *d, const QString &charset ) : KaffeineDVBsection( anum, tnum, charset )
 {
 	channels.setAutoDelete( true );
 	transponders.setAutoDelete( true );
@@ -346,8 +341,6 @@ DVBsi::DVBsi( bool *ok, int anum, int tnum, DvbStream *d, const QString &charset
 	adapter = anum;
 	tuner = tnum;
 	ns = 0;
-	pf[0].fd = fdDemux;
-	pf[0].events = POLLIN;
 }
 
 
@@ -384,8 +377,10 @@ bool DVBsi::tableSDT( unsigned char* buf )
 	ChannelDesc *desc;
 	QString s;
 	unsigned short tsid;
+	unsigned short nid;
 
-	tsid = getBits(buf,24,16);
+	tsid = getBits(buf+3,0,16);
+	nid = getBits(buf+8,0,16);
 	length = getBits(buf,12,12);
 	length -=8;
 	buf +=11;
@@ -394,6 +389,7 @@ bool DVBsi::tableSDT( unsigned char* buf )
 		desc = new ChannelDesc();
 		channels.append( desc );
 		desc->tp.tsid = tsid;
+		desc->tp.nid = nid;
 		desc->sid = getBits(buf,0,16);
 		//desc->fta = getBits(buf,27,1 );
 		loop = getBits(buf,28,12);
@@ -422,12 +418,13 @@ bool DVBsi::tablePMT( unsigned char* buf )
 	int type;
 	int sid;
 	int i;
-	int nlang=0;
 	int pid=0;
 	ChannelDesc *desc=0;
 	QString lang;
 	int ns;
 	unsigned char st;
+	bool audio, ac3;
+
 
 	sid = getBits(buf+3,0,16);
 	for ( i=indexChannels; i<(int)channels.count(); i++ ) {
@@ -458,6 +455,7 @@ bool DVBsi::tablePMT( unsigned char* buf )
 	}
 
 	while ( length>4 ) {
+		audio=ac3=false;
 		type = getBits(buf,0,8);
 		pid = getBits(buf,11,13);
 		if ( type==1/*mpeg1*/ || type==2/*mpeg2*/ || type==16/*mpeg4*/ || type==27/*h264*/ ) {
@@ -466,13 +464,7 @@ bool DVBsi::tablePMT( unsigned char* buf )
 			desc->vType = type;
 		}
 		if ( type==3 || type==4 ) {
-			if ( !desc->vpid )
-				desc->type=2;
-			nlang++;
-			if ( desc->napid<desc->maxapid ) {
-				desc->apid[(int)desc->napid].pid = pid;
-				desc->napid++;
-			}
+			audio = true;
 		}
 		loop = getBits(buf,28,12);
 		buf +=5;
@@ -484,9 +476,6 @@ bool DVBsi::tablePMT( unsigned char* buf )
 					break;
 				case 0x0A :
 					lang = langDesc( buf );
-					if ( (type==3 || type==4) && nlang<=desc->maxapid ) {
-						desc->apid[desc->napid-1].lang = lang;
-					}
 					break;
 				case 0x56 :
 					if ( type==6 )
@@ -510,12 +499,8 @@ bool DVBsi::tablePMT( unsigned char* buf )
 					}
 					break;
 				case 0x6a :
-					if ( type==6 && desc->napid<desc->maxapid ) {
-						desc->apid[(int)desc->napid].pid = pid;
-						desc->apid[(int)desc->napid].ac3 = 1;
-						desc->napid++;
-						nlang++;
-					}
+					audio = true;
+					ac3 = true;
 					break;
 				default :
 					break;
@@ -523,6 +508,16 @@ bool DVBsi::tablePMT( unsigned char* buf )
 			loop -=( getBits(buf,8,8)+2 );
 			buf +=( getBits(buf,8,8)+2 );
 		}
+		if ( audio && desc->napid<desc->maxapid ) {
+			if ( !desc->vpid )
+				desc->type=2;
+			desc->apid[(int)desc->napid].pid = pid;
+			if ( ac3 )
+				desc->apid[(int)desc->napid].ac3 = 1;
+			if ( !lang.isEmpty() )
+				desc->apid[(int)desc->napid].lang = lang;
+			desc->napid++;
+		}
 	}
 
 	return true;
@@ -532,10 +527,11 @@ bool DVBsi::tablePMT( unsigned char* buf )
 
 bool DVBsi::tablePAT( unsigned char *buf )
 {
-	int length, i;
-	int sid;
-	int pmt;
+	int length, i, sid, tsid, pmt;
+	bool got;
+	ChannelDesc *desc;
 
+	tsid = getBits(buf+3,0,16);
 	length = getBits(buf,12,12);
 	length -=5;
 	buf +=8;
@@ -545,12 +541,22 @@ bool DVBsi::tablePAT( unsigned char *buf )
 		pmt = getBits(buf,19,13);
 		buf +=4;
 		length -=4;
+		got = false;
 		for ( i=indexChannels; i<(int)channels.count(); i++ ) {
 			if ( channels.at( i )->sid==sid ) {
 				channels.at( i )->pmtpid = pmt;
+				got = true;
 				break;
 			}
 		}
+		if ( !got  && sid!=0 ) {
+			desc = new ChannelDesc();
+			channels.append( desc );
+			desc->tp.tsid = tsid;
+			desc->sid = sid;
+			desc->pmtpid = pmt;
+			desc->name = QString("TSID:%1-SID:%2").arg(tsid).arg(sid);
+		}
 	}
 
 	return true;
@@ -639,7 +645,7 @@ void DVBsi::stop()
 		return;
 
 	isRunning = false;
-	if ( !wait(2000) ) {
+	if ( !wait(10000) ) {
 		if ( ns ) {
 			ns->stop();
 			delete ns;
@@ -702,7 +708,7 @@ void DVBsi::run()
 	int i, j=0, k=0;
 	ChannelDesc chan;
 	Transponder trans;
-	unsigned short tsid;
+	unsigned short tsid, nid;
 	bool nitEnded=true, ok;
 
 	progressTransponder = 0;
@@ -760,9 +766,11 @@ void DVBsi::run()
 				}
 				getSection( channels.at( j )->pmtpid, 0x02, 5000, channels.at( j )->sid );    //PMTs
 				tsid = channels.at(j)->tp.tsid;
+				nid = channels.at(j)->tp.nid;
 				channels.at(j)->tp = *transponders.at(i);
 				if ( channels.at(j)->tp.tsid==0 )
 					channels.at(j)->tp.tsid = tsid;
+				channels.at(j)->tp.nid = nid;
 				channels.at(j)->tp.snr = dvb->getSNR();
 				channels.at(j)->completed = 1;
 			}
@@ -799,8 +807,10 @@ void DVBsi::run()
 			}
 			getSection( channels.at( j )->pmtpid, 0x02, 5000, channels.at( j )->sid );    //PMTs
 			tsid = channels.at(j)->tp.tsid;
+			nid = channels.at(j)->tp.nid;
 			channels.at(j)->tp = trans;
 			channels.at(j)->tp.tsid = tsid;
+			channels.at(j)->tp.nid = nid;
 			channels.at(j)->tp.snr = dvb->getSNR();
 			channels.at(j)->completed = 1;
 		}
