@@ -65,7 +65,7 @@ public :
 	bool tuneDvb( ChannelDesc *chan, bool dvr=true );
 	void stopFrontend();
 	virtual void run();
-	int goLive( ChannelDesc *chan, const QString &pipeName, int &napid );
+	int goLive( ChannelDesc *chan, const QString &pipeName );
 	void preStopLive();
 	void stopLive( ChannelDesc *chan );
 	void stop();
@@ -77,8 +77,9 @@ public :
 	bool running() const;
 	ChannelDesc getLiveChannel();
 	Transponder getCurrentTransponder();
+	unsigned long getCurrentFreq();
 	bool isTuned();
-	bool startTimer( ChannelDesc *chan, QString path, int maxsize, RecTimer *t );
+	bool startTimer( ChannelDesc *chan, QString path, int maxsize, RecTimer *t, QString filename );
 	int canStartTimer( bool &live, ChannelDesc *chan );
 	int canStartBroadcast( bool &live, ChannelDesc *chan );
 	bool startBroadcast( QPtrList<ChannelDesc> *list, Ts2Rtp *rtp );
@@ -113,7 +114,7 @@ private :
 	double getAzimuth( double angle );
 	double getSourceAngle( QString source );
 	int getSatPos( const QString &src );
-	bool setPids( DVBout *o, int napid=0 );
+	bool setPids( DVBout *o );
 	void removePids( DVBout *o );
 	void removeOut( DVBout *o );
 	void recordingState();
