@@ -1,7 +1,7 @@
 /*
  * scandialog.cpp
  *
- * Copyright (C) 2004-2006 Christophe Thommeret <hftom@free.fr>
+ * Copyright (C) 2004-2007 Christophe Thommeret <hftom@free.fr>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -71,7 +71,7 @@ ScanDialog::ScanDialog( QPtrList<DvbStream> *d, QPtrList<ChannelDesc> *ch, QSize
 		tmp = dvb->at(i)->getSources( true );
 		for ( j=0; j<(int)tmp.count(); j++ ) {
 			if ( tmp[j].startsWith( "AUTO" ) )
-				list.append( QString("AUTO(%1:%2)").arg(dvb->at(i)->adapter).arg(dvb->at(i)->tuner) );
+				list.append( QString("AUTO(%1:%2)").arg(dvb->at(i)->getAdapter()).arg(dvb->at(i)->getTuner()) );
 			else
 				list.append( tmp[j] );
 		}
@@ -92,7 +92,7 @@ ScanDialog::ScanDialog( QPtrList<DvbStream> *d, QPtrList<ChannelDesc> *ch, QSize
 	statLayout->addWidget( lab, 1, 0 );
 	snr = new KGradProgress( statusFrame );
 	statLayout->addWidget( snr, 1, 1 );
-	lab = new QLabel( i18n("Lock:"), statusFrame );
+	lab = new QLabel( i18n("Tuned:"), statusFrame );
 	statLayout->addWidget( lab, 2, 0 );
 	lock = new KLed( statusFrame, "lockLed" );
 	lock->setState( KLed::Off );
@@ -145,7 +145,7 @@ ScanDialog::ScanDialog( QPtrList<DvbStream> *d, QPtrList<ChannelDesc> *ch, QSize
 	else
 		offsetGroup->hide();
 	bool ok=false;
-	dvbsi = new DVBsi( &ok, ds->adapter, ds->tuner, ds, defaultCharset );
+	dvbsi = new DVBsi( &ok, ds->getAdapter(), ds->getTuner(), ds, defaultCharset );
 	connect( ds, SIGNAL(snrStatus(int)), snr, SLOT(setProgress(int)) );
 	connect( ds, SIGNAL(signalStatus(int)), signal, SLOT(setProgress(int)) );
 	connect( ds, SIGNAL(lockStatus(bool)), this, SLOT(setLock(bool)) );
@@ -216,7 +216,7 @@ void ScanDialog::setDvb( int index )
 			delete dvbsi;
 			ds = dvb->at(i);
 			ds->setScanning( true );
-			dvbsi = new DVBsi( &ok, ds->adapter, ds->tuner, ds, defaultCharset );
+			dvbsi = new DVBsi( &ok, ds->getAdapter(), ds->getTuner(), ds, defaultCharset );
 			connect( ds, SIGNAL(snrStatus(int)), snr, SLOT(setProgress(int)) );
 			connect( ds, SIGNAL(signalStatus(int)), signal, SLOT(setProgress(int)) );
 			connect( ds, SIGNAL(lockStatus(bool)), this, SLOT(setLock(bool)) );
@@ -313,13 +313,13 @@ void ScanDialog::newChannel()
 	//int i;
 
 	//for ( i=0; i<(int)searchComb->count(); i++ ) list.append( searchComb->text(i) );
-	if ( ds->dvbType==FE_QPSK )
+	if ( ds->getType()==FE_QPSK )
 		list.append( searchComb->currentText() );
 	else
 		list = ds->getSources();
 
 	chandesc->append( new ChannelDesc( chan ) );
-	chan.tp.type = ds->dvbType;
+	chan.tp.type = ds->getType();
 	chan.num = chandesc->count();
 
 	ChannelEditor dlg( list, false, &chan, chandesc, this );
@@ -587,21 +587,21 @@ bool ScanDialog::getTransData()
 		for ( i=402; i<900; i+=8 ) {
 			if ( offset167m->isChecked() ) {
 				s = QString("T %1 8MHz AUTO AUTO AUTO AUTO AUTO AUTO").arg( (i*1000000)-167000 );
-				parseTp( s, ds->dvbType, "" );
+				parseTp( s, ds->getType(), "" );
 			}
 			if ( offset0->isChecked() ) {
 				s = QString("T %1 8MHz AUTO AUTO AUTO AUTO AUTO AUTO").arg( i*1000000 );
-				parseTp( s, ds->dvbType, "" );
+				parseTp( s, ds->getType(), "" );
 			}
 			if ( offset167p->isChecked() ) {
 				s = QString("T %1 8MHz AUTO AUTO AUTO AUTO AUTO AUTO").arg( (i*1000000)+167000 );
-				parseTp( s, ds->dvbType, "" );
+				parseTp( s, ds->getType(), "" );
 			}
 		}
 		return true;
 	}
 
-	switch ( ds->dvbType ) {
+	switch ( ds->getType() ) {
 		case FE_QPSK : s += "dvb-s/"; break;
 		case FE_QAM : s += "dvb-c/"; break;
 		case FE_OFDM : s += "dvb-t/"; break;
@@ -617,7 +617,7 @@ bool ScanDialog::getTransData()
 				continue;
 			if ( s.length()==0 )
 				continue;
-			parseTp( s, ds->dvbType, searchComb->currentText() );
+			parseTp( s, ds->getType(), searchComb->currentText() );
 		}
 		f.close();
 		return true;
@@ -635,7 +635,7 @@ void ScanDialog::checkDuplicateName( ChannelDesc *chan )
 	bool loop;
 
 	org = name = chan->name;
-	
+
 	do {
 		loop = false;
 		for ( i=0; i<(int)chandesc->count(); i++ ) {
@@ -650,6 +650,46 @@ void ScanDialog::checkDuplicateName( ChannelDesc *chan )
 	chan->name = name;
 }
 
+bool ScanDialog::checkChannUpdate( ChannelDesc *chan )
+{
+	int i, j;
+	AudioPid a;
+	SubPid s;
+	ChannelDesc *desc;
+
+	for ( i=0; i<(int)chandesc->count(); i++ ) {
+		desc = chandesc->at(i);
+		if ( chan->tp.source==desc->tp.source && chan->tp.freq==desc->tp.freq && chan->sid==desc->sid ) {
+			chan->category=desc->category;
+			chan->num=desc->num;
+			chan->name=desc->name;
+			chan->fta=desc->fta;
+			for ( j=0; j<chan->napid; j++ ) {
+				if ( desc->apid[0].pid==chan->apid[j].pid ) {
+					if ( j>0 ) {
+						a = chan->apid[0];
+						chan->apid[0]=chan->apid[j];
+						chan->apid[j]=a;
+					}
+					break;
+				}
+			}
+			for ( j=0; j<chan->nsubpid; j++ ) {
+				if ( desc->subpid[0].pid==chan->subpid[j].pid ) {
+					if ( j>0 ) {
+						s = chan->subpid[0];
+						chan->subpid[0]=chan->subpid[j];
+						chan->subpid[j]=s;
+					}
+					break;
+				}
+			}
+			chandesc->replace(i,new ChannelDesc(*chan));
+			return false;
+		}
+	}
+	return true;
+}
 
 
 void ScanDialog::addSelected()
@@ -670,23 +710,24 @@ void ScanDialog::addSelected()
 			dit = (DListViewItem*)dit->nextSibling();
 			continue;
 		}
+		if(checkChannUpdate(chan)){
+			checkDuplicateName( chan );
 
-		checkDuplicateName( chan );
-
-		chan->num = chandesc->count()+1;
-		chandesc->append( new ChannelDesc( *chan ) );
-		it = new QListViewItem( channelsList, chan->name );
-		if ( chan->type==1 ) {
-			if ( chan->fta )
-				it->setPixmap( 0, tvcPix );
-			else
-				it->setPixmap( 0, tvPix );
-		}
-		else {
-			if ( chan->fta )
-				it->setPixmap( 0, racPix );
-			else
-				it->setPixmap( 0, raPix );
+			chan->num = chandesc->count()+1;
+			chandesc->append( new ChannelDesc( *chan ) );
+			it = new QListViewItem( channelsList, chan->name );
+			if ( chan->type==1 ) {
+				if ( chan->fta )
+					it->setPixmap( 0, tvcPix );
+				else
+					it->setPixmap( 0, tvPix );
+			}
+			else {
+				if ( chan->fta )
+					it->setPixmap( 0, racPix );
+				else
+					it->setPixmap( 0, raPix );
+			}
 		}
 		odit = dit;
 		dit = (DListViewItem*)dit->nextSibling();
@@ -775,25 +816,25 @@ void ScanDialog::addFiltered()
 			dit = (DListViewItem*)dit->nextSibling();
 			continue;
 		}
+		if(checkChannUpdate(chan)){
+			checkDuplicateName( chan );
 
-		checkDuplicateName( chan );
-
-		chan->num = chandesc->count()+1;
-		chandesc->append( new ChannelDesc( *chan ) );
-		it = new QListViewItem( channelsList, chan->name );
-		if ( chan->type==1 ) {
-			if ( chan->fta )
-				it->setPixmap( 0, tvcPix );
-			else
-				it->setPixmap( 0, tvPix );
+			chan->num = chandesc->count()+1;
+			chandesc->append( new ChannelDesc( *chan ) );
+			it = new QListViewItem( channelsList, chan->name );
+			if ( chan->type==1 ) {
+				if ( chan->fta )
+					it->setPixmap( 0, tvcPix );
+				else
+					it->setPixmap( 0, tvPix );
+			}
+			else {
+				if ( chan->fta )
+					it->setPixmap( 0, racPix );
+				else
+					it->setPixmap( 0, raPix );
+			}
 		}
-		else {
-			if ( chan->fta )
-				it->setPixmap( 0, racPix );
-			else
-				it->setPixmap( 0, raPix );
-		}
-
 		odit = dit;
 		dit = (DListViewItem*)dit->nextSibling();
 		delete odit;
