@@ -92,8 +92,9 @@ void Ts2Rtp::setSocket( const QString &addr, int m_port, int m_senderPort )
 
 bool Ts2Rtp::addChannels( QPtrList<ChannelDesc> *channels )
 {
-	int i, j, pmtpid=8191;
+	int i, j, k, pmtpid=8191;
 	ChannelDesc *desc, *d;
+	QValueList<int> pids;
 
 	if ( !rtpSocket && !makeSocket() )
 		return false;
@@ -101,8 +102,8 @@ bool Ts2Rtp::addChannels( QPtrList<ChannelDesc> *channels )
 	sendList = "";
 	for ( i=0; i<(int)channels->count(); i++ ) {
 		desc = channels->at( i );
-		sendList = sendList+desc->name+"|"+QString().setNum(desc->vpid)+"|"+QString().setNum(desc->apid[desc->ttpid].pid)+"|";
-		if ( desc->apid[desc->ttpid].ac3 ) sendList+= "y|";
+		sendList = sendList+desc->name+"|"+QString().setNum(desc->vpid)+"|"+QString().setNum(desc->apid[0].pid)+"|";
+		if ( desc->apid[0].ac3 ) sendList+= "y|";
 		else sendList+= "n|";
 		sendList+= QString().setNum(desc->subpid[0].pid);
 		sendList+= "|";
@@ -114,8 +115,15 @@ bool Ts2Rtp::addChannels( QPtrList<ChannelDesc> *channels )
 		sendList+= "|";
 		sendList+= desc->subpid[0].lang+"|";
 		for ( j=i; j<(int)channels->count(); j++ ) {
+			pids.clear();
 			d = channels->at( j );
-			while ( pmtpid==17 || pmtpid==d->vpid || pmtpid==d->apid[d->ttpid].pid || pmtpid==d->subpid[0].pid )
+			if ( d->vpid )
+				pids.append( d->vpid );
+			for ( k=0; k<d->napid && k<MAX_AUDIO; k++ )
+				pids.append( d->apid[k].pid );
+			for ( k=0; k<d->nsubpid && k<MAX_DVBSUB; k++ )
+				pids.append( d->subpid[k].pid );
+			while ( pmtpid==17 || pids.contains( pmtpid ) )
 				--pmtpid;
 		}
 		desc->pmtpid = pmtpid--;
@@ -256,7 +264,7 @@ void Ts2Rtp::setPSI()
 void Ts2Rtp::psiTables( QPtrList<ChannelDesc> *channels )
 {
 	unsigned char buf[15000];
-	int off, i, sectionOff, loopOff, descOff;
+	int off, i, j, sectionOff, loopOff, descOff, max;
 	int npack=0;
 	unsigned short tsid = channels->at(0)->tp.tsid;
 	ChannelDesc *desc;
@@ -345,52 +353,62 @@ void Ts2Rtp::psiTables( QPtrList<ChannelDesc> *channels )
 			buf[off++] = 0x00; buf[off++] = 0x00; // infos_length
 		}
 
-		if ( desc->apid[desc->ttpid].ac3 ) {
-			buf[off++] = 0x06; // stream type
-			buf[off++] = desc->apid[desc->ttpid].pid>>8; buf[off++] = desc->apid[desc->ttpid].pid&0xff;
-			buf[off++] = 0x00; buf[off++] = 0x09; // es info length
-			buf[off++] = 0x6a; // descriptor tag
-			buf[off++] = 0x01; // descriptor length
-			buf[off++] = 0x00;
+		for ( j=0; j<desc->napid && j<MAX_AUDIO; j++ ) {
+			if ( desc->apid[j].ac3 ) {
+				buf[off++] = 0x06; // stream type
+				buf[off++] = desc->apid[j].pid>>8; buf[off++] = desc->apid[j].pid&0xff;
+				buf[off++] = 0x00; buf[off++] = 0x09; // es info length
+				buf[off++] = 0x6a; // descriptor tag
+				buf[off++] = 0x01; // descriptor length
+				buf[off++] = 0x00;
+			}
+			else {
+				buf[off++] = 0x04; // stream type = audio
+				buf[off++] = desc->apid[j].pid>>8; buf[off++] = desc->apid[j].pid&0xff;
+				buf[off++] = 0x00; buf[off++] = 0x06; // es info length
+			}
+			buf[off++] = 0x0a; // iso639 descriptor tag
+			buf[off++] = 0x04; // descriptor length
+			if ( !desc->apid[j].lang.isEmpty() ) {
+				buf[off++] = desc->apid[j].lang.constref(0);
+				buf[off++] = desc->apid[j].lang.constref(1);
+				if ( desc->apid[j].ac3 )
+					buf[off++] = '_';
+				else
+					buf[off++] = desc->apid[j].lang.constref(2);
+			}
+			else if ( desc->apid[j].ac3 ) {
+				buf[off++] = 'd';
+				buf[off++] = 'd';
+				buf[off++] = 49+j;
+			}
+			else {
+				buf[off++] = 'c';
+				buf[off++] = 'h';
+				buf[off++] = 49+j;
+			}
+			buf[off++] = 0x00; // audio type
 		}
-		else {
-			buf[off++] = 0x04; // stream type = audio
-			buf[off++] = desc->apid[desc->ttpid].pid>>8; buf[off++] = desc->apid[desc->ttpid].pid&0xff;
-			buf[off++] = 0x00; buf[off++] = 0x06; // es info length
-		}
-		buf[off++] = 0x0a; // iso639 descriptor tag
-		buf[off++] = 0x04; // descriptor length
-		if ( !desc->apid[desc->ttpid].lang.isEmpty() ) {
-			buf[off++] = desc->apid[desc->ttpid].lang.constref(0);
-			buf[off++] = desc->apid[desc->ttpid].lang.constref(1);
-			buf[off++] = desc->apid[desc->ttpid].lang.constref(2);
-		}
-		else {
-			buf[off++] = '?';
-			buf[off++] = '?';
-			buf[off++] = '?';
-		}
-		buf[off++] = 0x00; // audio type
 
-		if ( desc->subpid[0].pid ) {
+		for ( j=0; j<desc->nsubpid && j<MAX_DVBSUB; j++ ) {
 			buf[off++] = 0x06; // stream type = ISO_13818_PES_PRIVATE
-			buf[off++] = desc->subpid[0].pid>>8; buf[off++] = desc->subpid[0].pid&0xff;
+			buf[off++] = desc->subpid[j].pid>>8; buf[off++] = desc->subpid[j].pid&0xff;
 			buf[off++] = 0x00; buf[off++] = 0x0a; // es info length
 			buf[off++] = 0x59; //DVB sub tag
 			buf[off++] = 0x08; // descriptor length
-			if ( !desc->subpid[0].lang.isEmpty() ) {
-				buf[off++] = desc->subpid[0].lang.constref(0);
-				buf[off++] = desc->subpid[0].lang.constref(1);
-				buf[off++] = desc->subpid[0].lang.constref(2);
+			if ( !desc->subpid[j].lang.isEmpty() ) {
+				buf[off++] = desc->subpid[j].lang.constref(0);
+				buf[off++] = desc->subpid[j].lang.constref(1);
+				buf[off++] = desc->subpid[j].lang.constref(2);
 			}
 			else {
-				buf[off++] = '?';
-				buf[off++] = '?';
-				buf[off++] = '?';
+				buf[off++] = 'c';
+				buf[off++] = 'h';
+				buf[off++] = 49+j;
 			}
-			buf[off++] = desc->subpid[0].type; //sub type
-			buf[off++] = desc->subpid[0].page>>8; buf[off++] = desc->subpid[0].page&0xff; // comp_page_id
-			buf[off++] = desc->subpid[0].id>>8; buf[off++] = desc->subpid[0].id&0xff; // anc_page_id
+			buf[off++] = desc->subpid[j].type; //sub type
+			buf[off++] = desc->subpid[j].page>>8; buf[off++] = desc->subpid[j].page&0xff; // comp_page_id
+			buf[off++] = desc->subpid[j].id>>8; buf[off++] = desc->subpid[j].id&0xff; // anc_page_id
 		}
 
 		buf[sectionOff-1] |= (off+1)>>8;
