@@ -68,7 +68,7 @@ int main(int argc, char *argv[])
 {
 	KAboutData aboutData("kaffeine", I18N_NOOP("Kaffeine Player"), KAFFEINE_VERSION,
 		I18N_NOOP("A media player for KDE 3. Can use multiple backends for playback, default (and recommended) is xine."),
-		KAboutData::License_GPL_V2, I18N_NOOP("Copyright (C) 2003-2006, The Kaffeine Authors"), 0,
+		KAboutData::License_GPL_V2, I18N_NOOP("Copyright (C) 2003-2007, The Kaffeine Authors"), 0,
 		"http://kaffeine.sourceforge.net");
 
 	aboutData.addAuthor("Christophe Thommeret", I18N_NOOP("Current maintainer"), "hftom@free.fr");
@@ -77,9 +77,10 @@ int main(int argc, char *argv[])
 
 	/* FIXME: what about the other contributors listed in CREDITS? --pfister */
 
+	aboutData.addCredit("Eldon Tyrell", I18N_NOOP("DVB patches."), "dr.e.tyrell@gmail.com");
 	aboutData.addCredit("Michael Hoertnagl", I18N_NOOP("Various patches."), "mtron@a1.net");
 	aboutData.addCredit("Ricardo Manuel Santos Rodrigues", I18N_NOOP("Various patches."), "madinfo@cadaval.net");
-	aboutData.addCredit("Christopher Martin", I18N_NOOP("Various valuable patches."), "christopher.martin@utoronto.ca");
+	aboutData.addCredit("Christopher Martin", I18N_NOOP("Various patches."), "christopher.martin@utoronto.ca");
 	aboutData.addCredit("Ben Jackson", I18N_NOOP("DVB OSD browsing patch."), "benj@puremourning.co.uk");
 	aboutData.addCredit("Rainer Wirtz", I18N_NOOP("DVB categories patches."), "rainer.wirtz@gmx.de");
 	aboutData.addCredit("Dieter Zander", I18N_NOOP("Logo for Kaffeine 0.8 and other artwork."), "dieter-mz@online.de");
